//***********************************************************************
// CM3CompassRose
// @Copyright HSU, Jim Graham, 2019
//***********************************************************************
CM3CompassRose = function () 
{
}
CM3CompassRose.prototype = Object.create( CMItem.prototype );

CM3CompassRose.prototype.constructor = CM3CompassRose;

//**************************************************************************************************
// Definitions
//**************************************************************************************************

// options for creating the faces for the points of a compass rose

CM3CompassRose.LEFT_FACES=0;
CM3CompassRose.RIGHT_FACES=1;
CM3CompassRose.ALL_FACES=2;

//**************************************************************************************************
// Private functions
//**************************************************************************************************
/**
* Creates a geometry for one set of points for a north arrow.  This can be the left facing or 
* right facing points.
*/
CM3CompassRose.GetPointGeometry=function(Faces,ArrowHeadLength,ArrowHeadWidth,ArrowBarbLength,Height)
{
	var x1=0;
	var y1=0;
	
	var ThePoint = new THREE.Geometry();
	
	ThePoint.vertices.push(
		new THREE.Vector3( x1, y1,  Height ), // 0, center
		
		new THREE.Vector3( x1,  y1+ArrowHeadLength,  0 ), // 1, top 
		new THREE.Vector3( x1+ArrowHeadLength,  y1,  0 ), // 2, right
		new THREE.Vector3( x1,  y1-ArrowHeadLength,  0 ), // 3, bottom
		new THREE.Vector3( x1-ArrowHeadLength,  y1,  0 ), // 4, left
		
		new THREE.Vector3(  x1+ArrowBarbLength, y1+ArrowBarbLength,  0 ), // 5, upper right 
		new THREE.Vector3(  x1+ArrowBarbLength, y1-ArrowBarbLength,  0 ), // 6, lower right
		new THREE.Vector3(  x1-ArrowBarbLength, y1-ArrowBarbLength,  0 ), // 7, lower left
		new THREE.Vector3(  x1-ArrowBarbLength, y1+ArrowBarbLength,  0 ) // 8, upper left
	);

	if ((Faces==CM3CompassRose.LEFT_FACES)||(Faces==CM3CompassRose.ALL_FACES))
	{
		ThePoint.faces.push( new THREE.Face3( 0, 1, 8 ) ); // center, upper left, top 
		ThePoint.faces.push( new THREE.Face3( 0, 2, 5 ) ); // 
		ThePoint.faces.push( new THREE.Face3( 0, 3, 6 ) );
		ThePoint.faces.push( new THREE.Face3( 0, 4, 7 ) );
	}
	if ((Faces==CM3CompassRose.RIGHT_FACES)||(Faces==CM3CompassRose.ALL_FACES))
	{
		ThePoint.faces.push( new THREE.Face3( 0, 5, 1 ) ); // center, top, upper right, 
		ThePoint.faces.push( new THREE.Face3( 0, 6, 2 ) );
		ThePoint.faces.push( new THREE.Face3( 0, 7, 3 ) );
		ThePoint.faces.push( new THREE.Face3( 0, 8, 4 ) );
	}
	ThePoint.computeBoundingSphere();
	ThePoint.computeFaceNormals ()

	return(ThePoint);
};
/**
* Creates an object for the points for a compass rose
* @private
*/
CM3CompassRose.GetPoints=function(LeftMaterial,RightMaterial,ArrowHeadLength,ArrowHeadWidth,ArrowBarbLength,Height)
{
	var ThePoint=new THREE.Object3D();
	
	// left faces
	var LeftFaces=CM3CompassRose.GetPointGeometry(CM3CompassRose.LEFT_FACES,ArrowHeadLength,ArrowHeadWidth,ArrowBarbLength,Height);

	var LeftPoint = new THREE.Mesh(LeftFaces, LeftMaterial);

	ThePoint.add(LeftPoint);

	// right faces
	var RightFaces=CM3CompassRose.GetPointGeometry(CM3CompassRose.RIGHT_FACES,ArrowHeadLength,ArrowHeadWidth,ArrowBarbLength,Height);

	var RightPoint = new THREE.Mesh(RightFaces, RightMaterial);

	ThePoint.add(RightPoint);
	
	return(ThePoint);
}
//**************************************************************************************************
// Public functions
//**************************************************************************************************
CM3CompassRose.GetCompassRose=function()
{
	var TheCompassRose=new THREE.Mesh();
	
	// add the points of the star on the rose
	
	var LeftMaterial=new THREE.MeshPhongMaterial({
		  color: 0xD4AF37, // 0xFFD700
		  side:THREE.FrontSide, 
		  opacity:1,
		  transparent :false
	});	

	var RightMaterial=new THREE.MeshPhongMaterial({
		  color: 0x555555,
		  side:THREE.FrontSide, 
		  opacity:1,
		  transparent :false
	});	
	
	var ThePoints=CM3CompassRose.GetPoints(LeftMaterial,RightMaterial,100,20,15,20);
	
	TheCompassRose.add(ThePoints);
	
	var ThePoints=CM3CompassRose.GetPoints(LeftMaterial,RightMaterial,60,20,15,20);
	
	var Angle=45*(Math.PI/180);
	
	ThePoints.rotateZ(Angle);
	
	TheCompassRose.add(ThePoints);
	
	// add a beveled curve (does not work well)
	
	var NumRows=60;
	var Radius=10;
	var NumColumns=20;
	
	var PathArray=CM3Utilities.GetArcArrayInDegrees(0,360,NumRows,120) // create the path for the extrusion to follow
	
	var CrossSectionArray=CM3Utilities.GetArcArrayInDegrees(0,360,NumColumns,Radius) // create the shape of the extrusion (start, end, segments, radius)

	var i=0;
	var TheExtrusion=new CM3PathExtrusion();
	TheExtrusion.SetPathArray(PathArray);
	TheExtrusion.SetCrossSectionArray(CrossSectionArray);
	TheExtrusion.SetSettings({"PathExtrusion":{"Closed":true}});
	TheExtrusion.ComputeGeometry();
	var TheSegment=TheExtrusion.GetGeometry();
		
	var LeftMaterial=new THREE.MeshPhongMaterial({
		  color: 0xD4AF37,
//				  side:THREE.FrontSide, 
		  side:THREE.DoubleSide, 
		  opacity:1,
		  transparent :true
	});	
//	var LeftMaterial = new THREE.MeshLambertMaterial( { color: 0x000000, wireframe: true } );
	
	var mesh = new THREE.Mesh( TheSegment, LeftMaterial );
	TheCompassRose.add(mesh);
	
//			edges = new THREE.VertexNormalsHelper( mesh, 20, 0x00ff00, 1 );
//			edges = new THREE.FaceNormalsHelper( mesh, 2, 0x00ff00, 1 );
//			TheCompassRose.add(edges);
	
/*	
	var CrossSectionArray=GetArcArrayInDegrees(180,360,8,10) // create the shape of the extrusion
	var PathArray=GetArcArrayInDegrees(0,360,8,120) // create the path for the extrusion to follow
	
	var PathCurve = new THREE.SplineCurve3(PathArray);
	PathCurve.type = 'catmullrom';
	PathCurve.closed = true;
	
	var extrudeSettings = {
		steps			: 100,
		bevelEnabled	: false,
		extrudePath		: PathCurve
	};

	var CrossSectionShape = new THREE.Shape( CrossSectionArray );
	var geometry = new THREE.ExtrudeGeometry( CrossSectionShape, extrudeSettings );
	var material = new THREE.MeshLambertMaterial( { color: 0xb00000, wireframe: false } );
	var mesh = new THREE.Mesh( geometry, LeftMaterial );
	TheCompassRose.add( mesh );
*/
	// add text

	var TheText=new CM3Text("N",0,150,0,TheCompassRose);
	
	return(TheCompassRose);
};
