/**
* Base class for controls that move objects in 3D space under user control.
* @module CM3Controls
* @Copyright HSU, Jim Graham, 2019
*/
CM3Controls=function() 
{
	//***************************************************************************
	// Settings
	//***************************************************************************
	this.Settings={}; 
	this.Settings.CM3Controls=	
	{
		MinElevation:0,
		MaxElevation:100000,
		
		MouseActive:true,
		KeysActive:true,
		TouchActive:true,
	};

	//***************************************************************************
	// Protected CM3Control Functions
	//***************************************************************************
	this.Initialize = function( TheObject, domElement ) 
	{
		// this is the object that is being moved by the controls (typically a camera)
		this.TheObject = TheObject;
	
		// dom element the scene is displayed in (typically the Canvas)
		this.domElement = ( domElement !== undefined ) ? domElement : document;

		// force an update at start
	
		this.Update();
	}

	/**
	* Called by event handlers after settings have changed to update the position and orientation
	* of the camera.
	*/
	this.Update = function() 
	{
		return(false);
	}
	/**
	* Sets the camera to the specified x,y,z location in the 3D scene.
	* @public
	*/
	this.SetPosition = function(x,y,z)
	{
		console.log("CM3Controls.SetPosition() needs to be overriden");
	}
	/**
	* Gets the position of the camera in world coordinates
	* @public
	*/
	this.GetPosition = function()
	{
		console.log("CM3Controls.GetPosition() needs to be overriden");
	}
	this.SetRotation = function(x,y,z)
	{
		console.log("CM3Controls.SetRotation() needs to be overriden");
	}
	this.LookAt = function(x,y,z)
	{
		console.log("CM3Controls.LookAt() needs to be overriden");
	}

	//*************************************************************************
	// Public CM3Controls Functions
	//*************************************************************************
	this.ZoomIn=function()
	{
	}
	this.ZoomOut=function()
	{
	}
	//*************************************************************************
	// Public CM3Controls event handlers
	//*************************************************************************
	//
	// event handlers - FSM: listen for events and reset state
	//

	this.MouseDown=function ( event )  
	{
		return(false);
	}

	this.MouseMove=function ( event ) 
	{
		return(false);
	}

	this.MouseUp=function ( event ) 
	{
		return(false);
	}

	this.MouseWheel=function ( event ) 
	{
		return(false);
	}

	this.KeyDown=function ( event ) 
	{
		return(false);
	}
	this.KeyUp=function ( event ) 
	{
		return(false);
	}

	this.TouchStart=function ( event )  
	{
		return(false);
	}

	this.TouchMove=function ( event ) 
	{
		return(false);
	}

	this.TouchEnd=function ( event )  
	{
		return(false);
	}


};

CM3Controls.prototype = Object.create( CMBase.prototype );
CM3Controls.prototype.constructor = CM3Controls;
//*************************************************************************
// CMBase Functions
//*************************************************************************
CM3Controls.prototype.CMBase_GetSettingsDefinitions=CMBase.prototype.GetSettingsDefinitions; // save the superclass function

CM3Controls.prototype.GetSettingsDefinitions=function() 
{
	var Result=this.CMBase_GetSettingsDefinitions(); // get the settings definitions from the superclass
	
	for (Key in CM3Controls.SettingDefintions)
	{
		Result[Key]=CM3Controls.SettingDefintions[Key];
	}

	return(Result); 
}

//******************************************************************
// Definitions
//******************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/
CM3Controls.SettingDefintions=
{
	CM3Controls:
	{
		MinElevation: { Name:"Min Elevation",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
		MaxElevation: { Name:"Max Elevation",Type:CMBase.DATA_TYPE_FLOAT, Default:1000 },
		
		MouseActive: { Name:"Mouse Active",Type:CMBase.DATA_TYPE_BOOLEAN,Default:true },
		KeysActive: { Name:"Keys Active",Type:CMBase.DATA_TYPE_BOOLEAN,Default:true },
		TouchActive: { Name:"Touch Active",Type:CMBase.DATA_TYPE_BOOLEAN,Default:true },
	},
};