/**
* Control class to make an TheObject fly like an airplane
*
* This is a heavily modified version of the THREE.FlyControls class
*
* @author James Baicoianu / http://www.baicoianu.com/
*
* Looks like this class allows an OGL TheObject to fly using keyboard controls.
* 
* @module CM3ControlsAirplane
* @Copyright HSU, Jim Graham, 2019
*/

/**
* TheObject contains the initial position of the TheObject
*/
CM3ControlsAirplane = function (  ) 
{
	CM3Controls.call(this);
	
	//***************************************************************************
	// Private properties
	//***************************************************************************
	
	this.tmpQuaternion = new THREE.Quaternion();

	this.moveState = { up: 0, right: 0, forward: 0, pitch: 0, yaw: 0, roll: 0 }; 

	//***************************************************************************
	// Settings
	//***************************************************************************
	this.Settings={}; // jjg remove when inherited
	this.Settings.CM3ControlsAirplane=	
	{
		MinElevation:0,
		MaxElevation:100000,
		
		MinPitch:-1,
		MaxPitch:0.2,
		MinRoll:-0.25,
		MaxRoll:0.25,
		MouseActive:true
	};
	this.movementSpeed = 30.0;
	this.rollSpeed = 0.05;

	// not sure if we need these
	this.dragToLook = false; // jjg I don't know what this is
	this.mouseStatus = 0; // or this

	//***************************************************************************
	// Private functions
	//***************************************************************************
	/**
	* Returns the dimensions of the dom element or the window if the dom element was not specified
	*/
	this.getContainerDimensions = function() 
	{
		var Result=null;
		
		if ( this.domElement != document ) {

			Result= {
				size	: [ this.domElement.offsetWidth, this.domElement.offsetHeight ],
				offset	: [ this.domElement.offsetLeft,  this.domElement.offsetTop ]
			};

		} else 
		{

			Result= {
				size	: [ window.innerWidth, window.innerHeight ],
				offset	: [ 0, 0 ]
			};

		}
		return(Result);
	};

	//***************************************************************************
	// Public CM3Control Functions
	//***************************************************************************
	this.Initialize = function( TheObject, domElement ) 
	{
		this.TheObject = TheObject; // the TheObject that we are flying, typically the camera
	
		this.domElement = domElement;
	}
	/**
	* Called by the view to update the 3D content based on the amount of 
	* time that has passed.
	* delta - how much time has passed since the last update
	*/
	this.Update = function( delta ) {

		var moveMult = delta * this.movementSpeed;
		var rotMult = delta * this.rollSpeed;

		// move the TheObject based on the movement vectors.  This is in the current direction of the TheObject
		
		this.TheObject.translateX( this.moveState.right * moveMult );
		this.TheObject.translateY( this.moveState.up * moveMult );
		this.TheObject.translateZ( this.moveState.forward * moveMult );

		var MinElevation=this.GetSetting("CM3Controls","MinElevation",0);
		var MaxElevation=this.GetSetting("CM3Controls","MaxElevation",10000000);
		
		// 
		
		//console.log( 'move:', [ this.moveVector.x, this.moveVector.y, this.moveVector.z ] );
		//console.log( 'rotate:', [ this.rotationVector.x, this.rotationVector.y, this.rotationVector.z ] );
		
		this.tmpQuaternion.set( this.moveState.pitch * rotMult, this.moveState.yaw * rotMult, this.moveState.roll * rotMult, 1 ).normalize();
		this.TheObject.quaternion.multiply( this.tmpQuaternion );

		// expose the rotation vector for convenience
		this.TheObject.rotation.setFromQuaternion( this.TheObject.quaternion, this.TheObject.rotation.order );

		var MinRoll=this.GetSetting("CM3ControlsAirplane","MinRoll",0);
		var MaxRoll=this.GetSetting("CM3ControlsAirplane","MaxRoll",0);
		var MinPitch=this.GetSetting("CM3ControlsAirplane","MinPitch",0);
		var MaxPitch=this.GetSetting("CM3ControlsAirplane","MaxPitch",0);
		
/*		if (this.TheObject.rotation.x<MinPitch) this.TheObject.rotation.x=MinPitch;
		if (this.TheObject.rotation.x>MaxPitch) this.TheObject.rotation.x=MaxPitch;
		
		if (this.TheObject.rotation.z<MinRoll) this.TheObject.rotation.z=MinRoll;
		if (this.TheObject.rotation.z>MaxRoll) this.TheObject.rotation.z=MaxRoll;
*/		
		//console.log( 'TheObject rotation:', [ this.TheObject.rotation.x, this.TheObject.rotation.y, this.TheObject.rotation.z ] );

		if (this.TheObject.position.y<MinElevation) this.TheObject.position.y=MinElevation;
		if (this.TheObject.position.y>MaxElevation) this.TheObject.position.y=MaxElevation;
		
//		console.log( 'TheObject position:', [ this.TheObject.position.x, this.TheObject.position.y, this.TheObject.position.z ] );

	};

	this.GetPosition=function() 
	{ 
		var Result=new THREE.Vector3(this.TheObject.position.x,this.TheObject.position.z,this.TheObject.position.y);
		return(Result); 
	}
	this.SetPosition=function(x,y,z)
	{
		this.TheObject.position.set(x,y,z); // 
		this.TheObject.updateProjectionMatrix();
		this.Update();
	}
	this.SetRotation = function(x,y,z)
	{
		this.TheObject.rotation.set(x,y,z);
	}
	this.LookAt = function(x,y,z)
	{
		this.TheObject.lookAt(x,y,z);
	}


	//***************************************************************************
	// Public Event handlers
	//***************************************************************************

	this.MouseDown = function( event ) 
	{
		var Used=false;
		
		var MouseActive=this.GetSetting("CM3ControlsAirplane","MouseActive",true);

		if (MouseActive)
		{
			if ( this.domElement !== document ) {
	
				this.domElement.focus();
			}
	
			if ( this.dragToLook )
			{
				this.mouseStatus ++;
			} 
			else 
			{
				switch ( event.button ) 
				{
					case 0: 
						this.moveState.forward = -1; 
						break;
					case 2: 
						this.moveState.forward = 1; 
						break;
				}
			}
			Used=true;
		}

		return(Used);

	};

	this.MouseMove = function( event ) 
	{
		var Used=false;
		
		var MouseActive=this.GetSetting("CM3ControlsAirplane","MouseActive",true);

		if (MouseActive)
		{
			if ( ! this.dragToLook || this.mouseStatus > 0 ) {
	
				var container = this.getContainerDimensions();
				var halfWidth  = container.size[ 0 ] / 2;
				var halfHeight = container.size[ 1 ] / 2;
	
				this.moveState.yaw   = - ( ( event.pageX - container.offset[ 0 ] ) - halfWidth  ) / halfWidth;
				this.moveState.pitch =   -( ( event.pageY - container.offset[ 1 ] ) - halfHeight ) / halfHeight;
				
				Used=true;
			}
		}

		return(Used);
	};

	this.MouseUp = function( event ) 
	{
		var Used=false;
		
		var MouseActive=this.GetSetting("CM3ControlsAirplane","MouseActive",true);

		if (MouseActive)
		{
			if ( this.dragToLook ) 
			{
				this.mouseStatus --;
	
				this.moveState.yaw =0;
				this.moveState.pitch = 0;
			} 
			else 
			{
				switch ( event.button ) 
				{
				case 0: 
					this.moveState.forward = 0; 
					break;
				case 2: 
					this.moveState.forward = 0; 
					break;
				}
			}
				
			Used=true;
		}
		return(true);
	};
	this.MouseWheel = function( TheEvent ) 
	{
		var Used=false;
		
		var MouseActive=this.GetSetting("CM3ControlsAirplane","MouseActive",true);

		if (MouseActive)
		{
			if ( this.dragToLook ) 
			{
				this.mouseStatus --;
	
				this.moveState.yaw =0;
				this.moveState.pitch = 0;
			} 
			else 
			{
				// get the amount the wheel has moved
				var Delta=TheEvent.detail? TheEvent.detail*(-120) : TheEvent.wheelDelta //delta returns +120 when wheel is scrolled up, -120 when scrolled down

				if (Delta>0)
				{
					this.moveState.forward = -1; 
				}
				else
				{
					this.moveState.forward = 1; 
				}
			}
				
			Used=true;
		}
		return(true);
	}
	
	/**
	*
	*/
	this.KeyDown=function ( event ) 
	{
		if ( event.altKey ) {
			return;
		}

		switch ( event.keyCode ) {

			case 16: /* shift */ this.movementSpeedMultiplier = .1; break;

			case 87: /*W*/ this.moveState.forward = -1; break;
			case 83: /*S*/ this.moveState.forward = 1; break;

			case 65: /*A*/ this.moveState.right = -1; break;
			case 68: /*D*/ this.moveState.right = 1; break;

			case 82: /*R*/ this.moveState.up = 1; break;
			case 70: /*F*/ this.moveState.up = -1; break;

			case 38: /*up*/ this.moveState.pitch = 1; break;
			case 40: /*down*/ this.moveState.pitch = -1; break;

			case 37: /*left*/ this.moveState.yaw = -1; break;
			case 39: /*right*/ this.moveState.yaw = 1; break;

			case 81: /*Q*/ this.moveState.roll = -0.2; break;
			case 69: /*E*/ this.moveState.roll = 0.2; break;

		}
	};

	this.KeyUp = function( event ) {

		switch ( event.keyCode ) {

			case 16: /* shift */ this.movementSpeedMultiplier = 1; break;

			case 87: /*W*/ this.moveState.forward = 0; break;
			case 83: /*S*/ this.moveState.forward = 0; break;

			case 65: /*A*/ this.moveState.right = 0; break;
			case 68: /*D*/ this.moveState.right = 0; break;

			case 82: /*R*/ this.moveState.up = 0; break;
			case 70: /*F*/ this.moveState.up = 0; break;

			case 38: /*up*/ this.moveState.pitch = 0; break;
			case 40: /*down*/ this.moveState.pitch = 0; break;

			case 37: /*left*/ this.moveState.yaw = 0; break;
			case 39: /*right*/ this.moveState.yaw = 0; break;

			case 81: /*Q*/ this.moveState.roll = 0; break;
			case 69: /*E*/ this.moveState.roll = 0; break;

		}
	};


};

CM3ControlsAirplane.prototype = Object.create( CM3Controls.prototype );
CM3ControlsAirplane.prototype.constructor = CM3ControlsAirplane;

//******************************************************************
// Definitions
//******************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/
CM3ControlsAirplane.SettingDefintions=
{
	CM3ControlsAirplane:
	{
		MinPitch: { Name:"Min Pitch",Type:CMBase.DATA_TYPE_FLOAT, Default:-1 },
		MaxPitch: { Name:"Max Pitch",Type:CMBase.DATA_TYPE_FLOAT, Default:0.2 },
		MinRoll: { Name:"Min Roll",Type:CMBase.DATA_TYPE_FLOAT, Default:-0.25 },
		MaxRoll: { Name:"Max Roll",Type:CMBase.DATA_TYPE_FLOAT, Default:0.25 },
	},
};