/**
* Class for manuiplating a view
* Some code taken from: mrdoob / http://mrdoob.com/
*
* The yawObject is actually the ViewingPlatform that moves in the scene.  The pitchObject 
* is placed on the yawObject so the camera can pitch forward and back.
* @module CM3ControlsHelicopter
* @Copyright HSU, Jim Graham, 2019
*/
//*****************************************************************************************************
// Constructor
//*****************************************************************************************************
CM3ControlsHelicopter=function ( ) 
{
	CM3Controls.call(this);
	
	//***********************************************************************************************
	// Private Definitions
	//***********************************************************************************************
	var MOVE_AMOUNT=50;
	var TURN_AMOUNT=Math.PI/100;
	
	var PI_2=Math.PI / 2;
	
	// definitions and local variable for what the user is doing with the mouse
	var STATE_PAN=1;
	var STATE_ROTATE=2;
	var STATE_ZOOM=3;

	//***********************************************************************************************
	// Private variables
	//***********************************************************************************************
	
	var yawObject; // the pitch object is placed on this object.  This object can turn left and right
	var pitchObject; // the camera is placed on this object.  This object can look up and down

	var Speed=0;
	
	// private properties used to manage the mouse down events
	var TheAnchorX=0; // location where the mouse was originally pressed down in pixel space
	var TheAnchorY=0;
	var OriginalPosition=0; // original position of the platform in 3d space
	
	var MouseState=null; // one of the STATE values: PAN,ROTATE, ZOOM
	
	var scope=this;
	
	//***************************************************************************
	// Settings
	//***************************************************************************
	
	this.Settings.CM3ControlsHelicopter=	
	{
		MinPitch:-1,
		MaxPitch:0.2,
		
		MinRoll:-0.25,
		MinRoll:0.25,
	};
	//***************************************************************************
	// Private functions
	//***************************************************************************
	/**
	* @private
	*/
	this.CheckRanges=function()
	{
		var MinPitch=this.GetSetting("CM3ControlsHelicopter","MinPitch");
		var MaxPitch=this.GetSetting("CM3ControlsHelicopter","MaxPitch");
		
		if (pitchObject.rotation.x<MinPitch) pitchObject.rotation.x=MinPitch;
		if (pitchObject.rotation.x>MaxPitch) pitchObject.rotation.x=MaxPitch;
		
		var MinElevation=this.GetSetting("CM3Controls","MinElevation");
		var MaxElevation=this.GetSetting("CM3Controls","MaxElevation");
		
		if (yawObject.position.y<MinElevation) yawObject.position.y=MinElevation;
		if (yawObject.position.y>MaxElevation) yawObject.position.y=MaxElevation;
	}
	//***********************************************************************************************
	// Object functions
	//***********************************************************************************************
	this.dispose=function() {

	};
	//***********************************************************************************************
	// CM3Control Functions
	//***********************************************************************************************

	this.Initialize=function( NewObject, domElement)
	{
		TheObject=NewObject;
		
		var TheScene=this.GetParent(CM3Scene);
		
		//TheOGLScene=TheScene.GetOGLScene();
		
		TheObject.position.set(0,0,0);
		
		//************************************************
		// setup the camera and platform
		
		// make sure the camera is not rotated
		TheObject.rotation.set( 0, 0, 0 );
	
		// the camera is mounted on the pitch object (think of the horiziontal bar of a gimble)
		pitchObject=new THREE.Object3D();
		pitchObject.add( TheObject );
	
		// the pitch bar is mounted on a yaw platform (think of the base of a theodolite)
		yawObject=new THREE.Object3D();
		yawObject.add( pitchObject ); 
	//	yawObject.lookAt(TheScene.GetPosition());	
		// set the default position to point to the center of the scene
	//	this.SetPosition(0,-800,600);
	//	this.SetPitch(1);
	//	this.SetYaw(0);
		yawObject.position.set(0,200,400); // is overriden when data loaded
		
		//***********************************************************************************************
		// Final construction steps.  These have to be here so they can call the functions above
		//***********************************************************************************************
		
		TheScene.AddOGLObject( yawObject );
			
		//************************************************
		
	}
	//***********************************************************************************************
	// CM3Control Event handlers
	//***********************************************************************************************
	/**
	* Handles mouse down events in the view
	* @private
	*/
	this.MouseDown=function(TheEvent)
	{
		var TheScene=this.GetParent(CM3Scene);
		
		TheEvent.preventDefault();
	
		{
			if ( TheEvent.button === THREE.MOUSE.LEFT ) 
			{
				MouseState=STATE_PAN;
			} 
			else if ( TheEvent.button === THREE.MOUSE.RIGHT ) 
			{
				MouseState=STATE_ROTATE;
			} 
			else if ( TheEvent.button === THREE.MOUSE.MIDDLE ) 
			{
				MouseState=STATE_ZOOM; //. ?
			}
			TheAnchorX=TheEvent.x;
			TheAnchorY=TheEvent.y;
			
			OriginalPosition=yawObject.position.clone();
		}
	}
	/**
	* Handles mouse movement events in the view
	* @private
	*/
	this.MouseMove=function(TheEvent)
	{
		var TheScene=this.GetParent(CM3Scene);
		
		if (MouseState!=null) // have an existing data of panning of rotating
		{
			switch (MouseState)
			{
			case STATE_PAN: // move the camera along the x,y axis
				if (this.ShiftDown)
				{
				}
				else
				{
					var ThePlatform=yawObject;
					var z=ThePlatform.position.z;
					var Factor=z/1000;
					
					var XDistance=(TheAnchorX-TheEvent.x)*Factor; // get the distance to move in pixel coordinates
					var YDistance=(TheAnchorY-TheEvent.y)*Factor;
					
					var Yaw=yawObject.rotation.z;
					
					var XNew=Math.sin(Yaw)*YDistance+Math.cos(Yaw)*XDistance; // compute the distance to move in ref coordinates
					var YNew=-Math.cos(Yaw)*YDistance+Math.sin(Yaw)*XDistance;
					
					yawObject.position.x=OriginalPosition.x+XNew;
					yawObject.position.y=OriginalPosition.y+YNew;
					
					this.CheckRanges();
				}
				break;
				
			case STATE_ROTATE: // rototate about the z (yaw) and x (pitch) axis
			
				// get the movement in the x and y direction independent of the browser
				
				var movementX=TheEvent.movementX || TheEvent.mozMovementX || TheEvent.webkitMovementX || 0;
				var movementY=TheEvent.movementY || TheEvent.mozMovementY || TheEvent.webkitMovementY || 0;
			
				yawObject.rotation.z += movementX * 0.002;
				
				var Temp=pitchObject.rotation.x-movementY * 0.002;
			
				pitchObject.rotation.x=Temp;
				
				this.CheckRanges();
//				pitchObject.rotation.x=Math.max( - PI_2, Math.min( PI_2, Temp ) );
				break;
			}	
		}
	};
	/**
	* @private
	*/
	this.MouseUp=function(TheEvent)
	{
		var TheScene=this.GetParent(CM3Scene);
		
		TheEvent.preventDefault();
	
//		if ( MouseOverThreeObject ) 
		{
//			SelectedOGLObject=null;
		}

	//	CanvasContainer.style.cursor='auto';
			
		MouseState=null;
	}
	/**
	* @private
	*/
	this.MouseWheel=function(TheEvent)
	{
		var delta=0;
		
		if ( TheEvent.wheelDelta !== undefined )
		{
			delta=TheEvent.wheelDelta; // WebKit / Opera / Explorer 9
		} 
		else if ( TheEvent.detail !== undefined ) 
		{
			delta=- TheEvent.detail; // Firefox
		}
		
		
		if (MouseState!=null)
		{
			switch (MouseState)
			{
			case STATE_PAN: // left mouse button down
				yawObject.translateZ( delta );
				break;
			case STATE_ROTATE:
				//pitchObject.rotateX( delta );
				break;
			default: // no butons down
//				yawObject.translateZ( delta );
				break;
			}
		}
		else // no buttons down
		{
			var CameraPosition=yawObject.position;
			var CameraZ=CameraPosition.z;
			
			var Factor=CameraZ*(-delta)/10000;
			
			var CameraDirection=TheObject.getWorldDirection(new THREE.Vector3());
			CameraDirection.multiplyScalar(Factor);
			
			yawObject.position.add(CameraDirection);
		}
	}
	/**
	* 
	* @private
	*/
	this.KeyDown=function( event )
	{
		switch ( event.keyCode ) 
		{
			case 38: // up
				this.SetPitch(this.GetPitch()-TURN_AMOUNT);
				break;
			case 40: // down
				this.SetPitch(this.GetPitch()+TURN_AMOUNT);
				break;
				
			case 37: // left
				this.SetYaw(this.GetYaw()+TURN_AMOUNT);
				break;
			case 39: // right
				this.SetYaw(this.GetYaw()-TURN_AMOUNT);
				break;
	
			case 87: // w forward
				var CameraDirection=TheObject.getWorldDirection(new THREE.Vector3());
				CameraDirection.multiplyScalar(10);
				yawObject.position.add(CameraDirection);
				this.CheckRanges();
				break;
			case 83: // s backward
				var CameraDirection=TheObject.getWorldDirection(new THREE.Vector3());
				CameraDirection.multiplyScalar(-10);
				yawObject.position.add(CameraDirection);
				this.CheckRanges();
				break;
				
			case 65: // a left
				yawObject.translateX(-10);
				break;
			case 68: // d right
				yawObject.translateX(10);
				break;
				
			case 87: // q roll
			case 69: // e roll
				break;
				
			case 82: // r up
				yawObject.position.y=yawObject.position.y+10;
					
				this.CheckRanges();
				break;
			case 70: // f down
				yawObject.position.y=yawObject.position.y-10;
					
				this.CheckRanges();
				break;
				
		}
	}
	this.KeyUp=function( event )
	{
	}
	//*****************************************************************************************************
	// CM3Control functions
	//*****************************************************************************************************

	/*
	* Zoom in to a higher resolution (map units are doubled relative to pixels)
	* @public
	*/
	this.ZoomIn=function()
	{
		var CameraDirection=TheObject.getWorldDirection();
		CameraDirection.multiplyScalar(100);
		yawObject.position.add(CameraDirection);
	}
	/*
	* Zoom in to a lower resolution (map units are halved relative to pixels)
	* @public
	*/
	this.ZoomOut=function()
	{
		var CameraDirection=TheObject.getWorldDirection();
		CameraDirection.multiplyScalar(-100);
		yawObject.position.add(CameraDirection);
	}
	//*****************************************************************************************************
	// CM3Control functions
	//*****************************************************************************************************

	this.Update=function() 
	{
	}
	
	this.Resize=function()
	{
		// the canvas aspect ratio is not correct unless we set the "width" and "height" of the element
		// rather than the style.  This must be done or it will display maps distorted
		
		var TheParent=TheCanvasElement.parentNode;
		var ParentWidth=jQuery(TheParent).width();
		var ParentHeight=jQuery(TheParent).height();
	
		TheRenderer.setSize( ParentWidth,ParentHeight ); // ths appears to reallocate the HTML canvas element
		
		TheObject.aspect=ParentWidth / ParentHeight;
		
		TheObject.updateProjectionMatrix ()
	}
	/**
	* Note that y and z are reversed in the functions below
	*/
	this.GetPosition=function() 
	{ 
		var Result=new THREE.Vector3(yawObject.position.x,yawObject.position.z,yawObject.position.y);
		return(Result); 
	}
	this.SetPosition=function(x,y,z)
	{
		try {
			yawObject.position.x=x;
			yawObject.position.y=y;
			yawObject.position.z=z;
		}
		catch(err) {
		  console.log("CM3ControlsHelicopter: "+err.message);
		  throw(err);
		}
		
		this.CheckRanges();
	}
	this.SetRotation = function(x,y,z)
	{
		this.SetPitch(x);
		this.SetYaw(y);
		if (z!=0) console.log("CM3ControlsHelicopter: SetRotation of z unfinished");
	}
	this.LookAt = function(x,y,z)
	{
		console.log("CM3ControlsHelicopter: LookAt unfinished");
	}


	//*****************************************************************************************************
	// Public CM3ControlsHelicopter settings functions
	//*****************************************************************************************************

	/**
	* @public
	*/
	this.GetPitch=function() { return(pitchObject.rotation.x); }
	this.SetPitch=function(NewPitch)
	{
		pitchObject.rotation.x=NewPitch;
		
		this.CheckRanges();
	}
	this.GetYaw=function() { return(yawObject.rotation.y); }
	this.SetYaw=function(NewYaw)
	{
		yawObject.rotation.y=NewYaw;
	}
	this.GetSpeed=function() { return(Speed); }
	this.SetSpeed=function(NewSpeed) { Speed=NewSpeed; }
	
}
CM3ControlsHelicopter.prototype=new CM3Controls();

CM3ControlsHelicopter.prototype.constructor=CM3ControlsHelicopter;

//******************************************************************
// Definitions
//******************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/
CM3ControlsHelicopter.SettingDefintions=
{
	CM3ControlsHelicopter:
	{
		MinPitch: { Name:"Min Pitch",Type:CMBase.DATA_TYPE_FLOAT, Default:-1 },
		MaxPitch: { Name:"Max Pitch",Type:CMBase.DATA_TYPE_FLOAT, Default:0.2 },
		MinRoll: { Name:"Min Roll",Type:CMBase.DATA_TYPE_FLOAT, Default:-0.25 },
		MaxRoll: { Name:"Max Roll",Type:CMBase.DATA_TYPE_FLOAT, Default:0.25 },
	},
};