/**************************************************************************************************
* CM3Geo Class
* Handles the geospatial display of data in a 3D scene.  
*
* @module CM3Geo
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
function CM3Geo(TheScene)
{
	CMGeo.call(this);
	
	this.SetParent(TheScene);
	
	this.OGLGeo=new THREE.Object3D();
	
	this.TranslateX=0;
	this.TranslateY=0;
	
	this.TheProjector=null;
}
CM3Geo.prototype=Object.create(CMGeo.prototype);

CM3Geo.prototype.constructor=CM3Geo;
//**************************************************************************************************
// CMGeo Functions
//**************************************************************************************************

/**
* Remove the layer at the specified index from the list of layers
* @public
* @param Index - 
*/
CM3Geo.prototype.CMGeo_DeleteLayer=CMGeo.prototype.DeleteLayer;

CM3Geo.prototype.DeleteLayer=function(Index) 
{
	var TheLayer=this.GetChild(Index);
	
	TheLayer.SetSetting("Item","Status","CMItem.STATUS_HIDDEN");
	
//	TheLayer.SetVisible(false); // have to make the layer invisible so it is removed from the OGL
	
	Result=this.CMGeo_DeleteLayer(Index);
	
	return(Result);
}

/**
* Similar to the CMScene MouseDown except recieves a RayTrace object instead of X,Y.
* Different from CM3Item because the control object is used.
* @public
*//*
CM3Geo.prototype.MouseDown=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
		
	var NumLayers=this.GetNumChildren();
	
	for (var i=0;(i<NumLayers)&&(Used==false);i++)
	{
		var TheChild=this.GetChild(i);
		
		Used=TheChild.MouseDown(TheView,TheEvent,TheRayTracer);
	}
	return(Used);
}*/
/**
* Similar to the CMScene MouseDown except recieves a RayTrace object instead of X,Y.
* Different from CM3Item because the control object is used.
* @public
*//*
CM3Geo.prototype.MouseMove=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
		
	var NumLayers=this.GetNumChildren();
	
	for (var i=0;(i<NumLayers)&&(Used==false);i++)
	{
		var TheChild=this.GetChild(i);
		
		Used=TheChild.MouseMove(TheView,TheEvent,TheRayTracer);
	}
	return(Used);
}*/
/**
* Similar to the CMScene MouseDown except recieves a RayTrace object instead of X,Y.
* Different from CM3Item because the control object is used.
* @public
*//*
CM3Geo.prototype.MouseUp=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
		
	var NumLayers=this.GetNumChildren();
	
	for (var i=0;(i<NumLayers)&&(Used==false);i++)
	{
		var TheChild=this.GetChild(i);
		
		Used=TheChild.MouseUp(TheView,TheEvent,TheRayTracer);
	}
	return(Used);
}
*/

//**************************************************************************************************
// CM3Geo Functions
//**************************************************************************************************
CM3Geo.prototype.GetOGLGeo=function()
{
	return(this.OGLGeo);
};
/**
* No longer used but left in case we want it in the future
*/
CM3Geo.prototype.SetTranslate=function(NewX,NewY)
{
	alert("Called CM3Geo.SetTranslate()");
	
	this.TranslateX=NewX;
	this.TranslateY=NewY;
	
	this.OGLGeo.translateY(this.TranslateY);
	this.OGLGeo.translateX(this.TranslateX);
};
/**
* No longer used but left in case we want it in the future
*/
CM3Geo.prototype.ZoomToBounds=function(Bounds)
{
	alert("Called CM3Geo.ZoomToBounds()");
	
	var Width=Bounds.XMax-Bounds.XMin;
	var Height=Bounds.YMax-Bounds.YMin;
	
	var Factor=Width;
	if (Height>Width) Factor=Height;
	
	var TheScale=1000/Factor;
	
	// find the distance to center the layers
	var XOffset=-(Bounds.XMax+Bounds.XMin)/2;
	var YOffset=-(Bounds.YMax+Bounds.YMin)/2;
	
	XOffset=XOffset*TheScale;
	YOffset=YOffset*TheScale;
	
	this.SetTranslate(XOffset,YOffset);
	
//	this.OGLGeo.scale.set(TheScale,TheScale,TheScale);
};
/**
* @protected
*/
CM3Geo.prototype.AddOGLObjectToGeo=function(TheObject)
{
	this.OGLGeo.add(TheObject);
};
/**
* @protected
*/
CM3Geo.prototype.RemoveOGLObjectFromGeo=function(TheObject)
{
	this.OGLGeo.remove(TheObject);
};

/**
* Set the projection for coordinate conversion
* @public
* @param TheProjector - the projector to use to project coordinates
*/
CM3Geo.prototype.ProjectFromGeographic=function(Long,Lat,Elevation)
{
	var TheProjector=this.GetProjector();
	var Result=null;
	
	if (TheProjector!=null) 
	{
		Result=TheProjector.ProjectFromGeographic(Long,Lat,Elevation);
	}
	else
	{
		Result=[Long,Lat,Elevation];
	}
 
	return(Result);
}
CM3Geo.prototype.GetNormalFromGeographic=function(Long,Lat)
{
	var TheProjector=this.GetProjector();
	var Result=null;
	
	if (TheProjector!=null) 
	{
		Result=TheProjector.GetNormalFromGeographic(Long,Lat,0);
	}
	else
	{
		Result=[0,0,0];
	}
 
	return(Result);
}
/*
CM3Geo.prototype.GetRotationsFromProjected=function(x,y,z)
{
	var TheProjector=this.GetProjector();
	var TheVector=null;
	
	if (TheProjector!=null) 
	{
		TheVector=TheProjector.GetRotationsFromProjected(x,y,z);
	}
	else
	{
		TheVector=new THREE.Vector3(0,0,0);
	}
 
	return(TheVector);
}
*/