/**************************************************************************************************
* CM3ItemOrigin Class
*
* @module CM3ItemOrigin
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************

/**
 * Container class for a 3D item that fits in a box (e.g. boxes, cones, cylinders, pyramids)
 */
function CM3ItemOrigin () 
{
	CM3Item.call(this);
	
	//
	
	this.TimeSlices[0].Settings=	
	{
	}

}
CM3ItemOrigin.prototype=Object.create(CM3Item.prototype);

CM3ItemOrigin.prototype.constructor=CM3ItemOrigin;
//**************************************************************************************************
// Definitions
//**************************************************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/
CM3ItemOrigin.SettingDefintions=
{
	Origin:
	{
		AxisLength: { Name:"Axis Length",Type:CMBase.DATA_TYPE_FLOAT, Default:100 },
	}
};

//**************************************************************************************************
// CMBase functions
//**************************************************************************************************
CM3ItemOrigin.prototype.CMItem_GetSettingsDefinitions=CMItem.prototype.GetSettingsDefinitions; // save the superclass function

CM3ItemOrigin.prototype.GetSettingsDefinitions=function() 
{
	var Result=this.CMItem_GetSettingsDefinitions(); // get the settings definitions from the superclass
	
	for (Key in CM3ItemOrigin.SettingDefintions)
	{
		Result[Key]=CM3ItemOrigin.SettingDefintions[Key];
	}

	return(Result); 
}

//**************************************************************************************************
// CM3Item functions
//**************************************************************************************************
/**
* Private function to initlaize the item's OpenGL object
*/
CM3ItemOrigin.prototype.Initialize=function()
{
	if (this.TheOGLObject==null)
	{
		this.SetOGLObjectVisible(false); // make the object invisible while we are changing things
		
		// setup the material
		
		this.TheMaterial=this.GetMaterial();
		
		if ((this.TheMaterial!=null))
		{
			this.TheOGLObject=new THREE.Group();
			
			//*********************************************
			// Add the objects
			
			var AxisLength=this.GetSetting("Origin","AxisLength",100); // this is all that has to be set
			var NumSides=20;
			
			var Factor=AxisLength/100;
			
			var AxisWidth=AxisLength*0.03;
			
			var ArrowWidth=AxisLength*0.05;
			var ArrowHeight=AxisLength*0.2;
			var ArrowNumSides=NumSides;

			var PyramidPosition=AxisLength*1.1;
			var TextPosition=AxisLength*1.25;
			
			var TextSettings = { 
				font: CM3Text.GlobalFonts[0],
				size: 12*Factor,
				height: 2*Factor,
				curveSegments: 10,
				bevelEnabled: true,  // beveled instead of 90 degree change
				bevelSegments: 5,  // number of bevel layers (more makes a smoother bevel but draws slower
				bevelSize: 0.1,  // width of bevel (distance from shape outline
				bevelThickness: 0.1 // how deep the bevel goes into the extrusion 
			};
			//***************************************
			// Add the objects
			
			// origin sphere
			TheGeometry = new THREE.SphereBufferGeometry( AxisLength/20,NumSides,NumSides );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			this.TheOGLObject.add(TheMesh);
			
			//***************************************
			// x-axis
			
			// line
			TheGeometry = new THREE.BoxBufferGeometry( AxisLength, AxisWidth, AxisWidth );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateX(AxisLength/2);
			this.TheOGLObject.add(TheMesh);
			
			// arrow
			TheGeometry = new THREE.CylinderBufferGeometry(0, ArrowWidth,ArrowHeight, NumSides );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateX(PyramidPosition);
			TheMesh.rotateZ(-Math.PI/2);
			this.TheOGLObject.add(TheMesh);

			// text
			TheGeometry=new THREE.TextBufferGeometry( "X", TextSettings);
			TheGeometry.computeBoundingBox();
			TheGeometry.computeVertexNormals();
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.rotateX(Math.PI/2);
			TheMesh.translateX(TextPosition);
	
			this.TheOGLObject.add(TheMesh);
			
			//***************************************
			// y-axis
			
			// line
			TheGeometry = new THREE.BoxBufferGeometry( AxisWidth, AxisLength, AxisWidth );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateY(AxisLength/2);
			this.TheOGLObject.add(TheMesh);
			
			// arrow
			TheGeometry = new THREE.CylinderBufferGeometry(0, ArrowWidth,ArrowHeight, NumSides );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateY(PyramidPosition);
			this.TheOGLObject.add(TheMesh);

			// text
			TheGeometry=new THREE.TextBufferGeometry( "Y", TextSettings);
			TheGeometry.computeBoundingBox();
			TheGeometry.computeVertexNormals();
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateY(TextPosition);
			TheMesh.rotateX(Math.PI/2);
			this.TheOGLObject.add(TheMesh);
			
			//***************************************
			// z-axis
			
			// line
			TheGeometry = new THREE.BoxBufferGeometry( AxisWidth, AxisWidth, AxisLength );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateZ(AxisLength/2);
			this.TheOGLObject.add(TheMesh);
			
			// arrow
			TheGeometry = new THREE.CylinderBufferGeometry(0, ArrowWidth,ArrowHeight, NumSides );
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateZ(PyramidPosition);
			TheMesh.rotateX(Math.PI/2);
			this.TheOGLObject.add(TheMesh);

			// text
			TheGeometry=new THREE.TextBufferGeometry( "Z", TextSettings);
			TheGeometry.computeBoundingBox();
			TheGeometry.computeVertexNormals();
			var TheMesh = new THREE.Mesh(TheGeometry, this.TheMaterial);
			TheMesh.translateZ(TextPosition);
			TheMesh.rotateX(Math.PI/2);
			this.TheOGLObject.add(TheMesh);
			
			//*********************************************
			// Add the objects
			
			var Rotation=this.GetSetting("Item","Rotation");
			
			if (Rotation!=undefined)
			{
				this.TheOGLObject.rotateX(Rotation[0]);
				this.TheOGLObject.rotateY(Rotation[1]);
				this.TheOGLObject.rotateZ(Rotation[2]);
			}
			this.TheOGLObject.position.copy( this.ThePosition );
			
			var Scale=this.GetSetting("Item","Scale");
			
			if (Scale!=undefined)
			{
				this.TheOGLObject.scale.set(Scale[0],Scale[1],Scale[2]);
			}
		}
		this.SetOGLObjectVisible(true); // make the object visible
	}
};

CM3ItemOrigin.prototype.GetGeometry=function()
{
	var TheGeometry=null;

	var AxisSettings={
		Style: {
			fillStyle:"rgba(128,128,128)",
		},
		Dimensions: {
			Width:AxisLength,
			Height:AxisWidth,
			Depth:AxisWidth,
		},
	}
	var AxisArrowSettings={
		Style: {
			fillStyle:"rgba(128,128,128)",
		},
		Dimensions: {
			Width:AxisLength*0.1,
			Height:AxisLength*0.2,
			NumSides:20,
		},
	}
	var FontSettings={
		Style: {
			fillStyle:"rgba(128,128,128)",
		},
		Extrusion: {
			bevelSize:.1,
			bevelThickness:.1,
		},
	}
	
	return(Geometries);
}

