/**************************************************************************************************
* CM3LayerLights Class
*
* @module CM3LayerLights
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
/*
* Constructor
* @public, @constructs
*/
function CM3LayerLights()
{
	CMLayer.call(this);
	
	this.SetName("Lights");
};

CM3LayerLights.prototype=Object.create(CMLayer.prototype); // inherit prototype functions from PanelBase()

CM3LayerLights.prototype.contructor=CM3LayerLights; // override the constructor to go to ours
//******************************************************************
// CMBase Functions
//******************************************************************

//jjg - fix this when we fix the SendMessageToDescendants() call in CMLayer
CM3LayerLights.prototype.SendMessageToDescendants=function(Message,AdditionalInfo)  
{ 
	var Used=false;
	
	var NumChildren=this.GetNumChildren();
	
	for (var i=0;(i<NumChildren)&&(Used==false);i++)
	{
		var TheChild=this.GetChild(i);
		
		Used=TheChild.SendMessageToDescendants(Message,AdditionalInfo);
	}
	return(Used);
}
CMItem.prototype.CMItem_SetSetting=CMItem.prototype.SetSetting;

CM3LayerLights.prototype.SetSetting=function(Group,Key,Value) 
{
	if ((Group=="Item")&&(Key=="Status"))
	{
		var Status=this.GetSetting(Group,Key);
	
		var CurrentFlag=true;
		if (Status==CMItem.STATUS_HIDDEN) CurrentFlag=false;
		
		var NewFlag=true;
		if (Value==CMItem.STATUS_HIDDEN) NewFlag=false;
		
		if (CurrentFlag!=NewFlag)
		{
			var NumChildren=this.GetNumChildren();
			
			for (var i=0;(i<NumChildren)&&(Used==false);i++)
			{
				var TheChild=this.GetChild(i);
				
				TheChild.SetOGLObjectVisible(NewFlag);
			}
		}
	}
	else 	
	{
		this.CMItem_SetSetting(Group,Key,Value);
	}
}

//**************************************************************************************************
// Public CM3LayerLights Functions
//**************************************************************************************************

CM3LayerLights.prototype.InsertLight=function(TheLight)
{
	this.AddChild(TheLight);
	
	return(TheLight);
}

CM3LayerLights.prototype.DeleteLight=function(TheLight)
{
	TheLight.SetOGLObjectVisible(false); // removes from selectable and TheOGLScene
	
	this.RemoveChild(TheLight);
	return(TheLight);
}

//**************************************************************************************************
// Public CMItem Functions
//**************************************************************************************************

//CM3LayerLights.prototype.CMLayer_SetOGLObjectVisible=CMLayer.prototype.SetOGLObjectVisible;

CM3LayerLights.prototype.SetOGLObjectVisible=function(Flag)
{
	//if (this.GetVisible()!=Flag)
	{
	//	this.CMLayer_SetOGLObjectVisible(Flag);
		
		for (var i=0;i<this.GetNumChildren();i++)
		{
			var TheLight=this.GetChild(i);
			
			TheLight.SetOGLObjectVisible(Flag);
		}
	}
}

//**************************************************************************************************
// Public CMItem Functions
//**************************************************************************************************

CM3LayerLights.prototype.CMLayer_FillPopupMenu=CMLayer.prototype.FillPopupMenu;

CM3LayerLights.prototype.FillPopupMenu=function(ThePopupMenu)
{
	this.CMLayer_FillPopupMenu(ThePopupMenu);
	
	if (true)
	{
		var AddLightMenuItem=document.createElement('div');
		AddLightMenuItem.className="CM_LayerListPopupMenuItem";
		AddLightMenuItem.setAttribute("id","CM_AddLight");
		AddLightMenuItem.innerHTML="Add Light";
		
		AddLightMenuItem.TheLayer=this;
		
		AddLightMenuItem.ThePopupMenu=ThePopupMenu;
		
		AddLightMenuItem.addEventListener('click', function(event)
		{
			this.ThePopupMenu.style.visibility="hidden";
			
			var TheLight=new CM3Light(CM3Light.TYPE_POINT);
		
			TheLight.SetSetting("Item","PositionOffset",[0,0,130]);
			TheLight.SetSetting("Style","fillStyle","rgb(128,128,128)");
			TheLight.SetSetting("ControlObject","Radius",8);
			this.TheLayer.InsertLight(TheLight);
				
			event.stopPropagation();
		});
		ThePopupMenu.appendChild(AddLightMenuItem);
	}
}
CM3LayerLights.prototype.Paint=function(TheView) 
{
	var NumLights=this.GetNumChildren();
	
	for (var i=0;i<NumLights;i++)
	{
		var TheLight=this.GetChild(i);
	
		TheLight.Paint(TheView);
	}
}
