/**************************************************************************************************
* CM3LayerSpecial Class
*
* @module CM3LayerSpecial
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
/*
* Constructor
* @public, @constructs
*/
function CM3LayerSpecial()
{
	CMLayer.call(this);

	this.TheObjects=[];
};

CM3LayerSpecial.prototype=Object.create(CMLayer.prototype); // inherit prototype functions from PanelBase()

CM3LayerSpecial.prototype.contructor=CM3LayerSpecial; // override the constructor to go to ours

//**************************************************************************************************
// Definitions
//**************************************************************************************************

var TheTextureLoader = new THREE.TextureLoader();

CM3LayerSpecial.FLOOR=1;
CM3LayerSpecial.SPHERICAL_EARTH=2;
CM3LayerSpecial.COMPASS_ROSE=3;
CM3LayerSpecial.ARROW=4;
CM3LayerSpecial.OBJ_FILE=5;

//**************************************************************************************************
// Public CMItem Functions
//**************************************************************************************************

//**************************************************************************************************
// Public CM3LayerSpecial Functions
//**************************************************************************************************

CM3LayerSpecial.prototype.AddSphericalEarth=function()
{
	var geometry   = new THREE.SphereGeometry(120.5, 32, 32) // sphere
	
	var material  = new THREE.MeshPhongMaterial()
	material.map    = TheTextureLoader.load('images/earthmap1k.jpg') // image
	material.bumpMap    = TheTextureLoader.load('images/earthbump1k.jpg') // "texture" of the image
	material.bumpScale = 10
	material.specularMap    = TheTextureLoader.load('images/earthspec1k.jpg') // reflectrance
	material.specular  = new THREE.Color('grey')
	
	var SphericalEarth = new THREE.Mesh(geometry, material)
	
	SphericalEarth.Type=CM3LayerSpecial.SPHERICAL_EARTH;

	var TheScene=this.GetParent(CM3Scene);
		
	TheScene.AddOGLObject(SphericalEarth)
	
	// add clouds
	
	var canvasCloud    =  TheTextureLoader.load('images/earthcloudmap.jpg') // 
	
	var geometry   = new THREE.SphereGeometry(125, 32, 32)
	var material  = new THREE.MeshPhongMaterial({
		map     	: canvasCloud,
		side        : THREE.DoubleSide,
		opacity     : 0.2,
		transparent : true,
		depthWrite  : false,
	})
	var cloudMesh = new THREE.Mesh(geometry, material) // move into item
	
	SphericalEarth.add(cloudMesh)
	
	//

	var TheScene=this.GetParent(CM3Scene);
		
	TheScene.AddMesh(SphericalEarth);
	
	this.TheObjects.push(SphericalEarth);
}

CM3LayerSpecial.prototype.AddFloor=function(Width,Height,X,Y,Z,Opacity)
{
	if (Opacity==undefined) Opacity=0.5;
	
	Y=-Y;
	
	var TheMainContainer=this.GetParent(CMMainContainer);
	var TheImageFolder=TheMainContainer.GetSetting("MainContainer","ImageFolder");
	
	var floorTexture =  TheTextureLoader.load( '/Archive/Images/2019/3DTextures/checkerboard.jpg' );
	
	floorTexture.wrapS = floorTexture.wrapT = THREE.RepeatWrapping; 
	floorTexture.repeat.set( 10, 10 );
	
	var floorMaterial = new THREE.MeshBasicMaterial( { 
		map: floorTexture, 
		side: THREE.DoubleSide,
		opacity:Opacity,
		transparent :true
	});	

	if (Width==undefined)
	{
		Width=1000;
		Height=1000;
	}
	var WidthSegments=10; // Width/100;
	var HeightSegments=10; // Height/100;
	
	var floorGeometry = new THREE.PlaneGeometry(Width, Height, WidthSegments, HeightSegments);
	
	TheFloor= new THREE.Mesh(floorGeometry, floorMaterial);
	
//	TheFloor.rotateX(Math.PI/2);
	if (X!=undefined)
	{
		TheFloor.position.set(X,Y,Z); // is overriden when data loaded
	}
	
	TheFloor.Type=CM3LayerSpecial.FLOOR;
	//

	var TheScene=this.GetParent(CM3Scene);
		
	TheScene.AddOGLObject(TheFloor);
	
	this.TheObjects.push(TheFloor);
	
	return(TheFloor);
}

CM3LayerSpecial.prototype.AddCompassRose=function()
{
	TheCompassRose=CM3CompassRose.GetCompassRose();
	
	TheCompassRose.Type=CM3LayerSpecial.COMPASS_ROSE;

	//

	var TheScene=this.GetParent(CM3Scene);
	
	var TheGeo=TheScene.GetGeo();
	
	TheGeo.AddOGLObjectToGeo(TheCompassRose);
	
	this.TheObjects.push(TheCompassRose);
}

CM3LayerSpecial.prototype.AddArrow=function()
{
	var x1=0;
	var y1=0;
	var z1=1;
	
	var ArrowHeadWidth=20;
	var ArrowHeadLength=20;
	var ArrowBodyLength=50;
	var ArrowBodyWidth=10;
	var ArrowBarbLength=8;
			
	var TheArrowShape = new THREE.Shape();
	
	TheArrowShape.moveTo( x1,y1 );
	TheArrowShape.lineTo( x1-ArrowHeadWidth,y1-ArrowHeadLength-ArrowBarbLength );
	TheArrowShape.lineTo( x1-ArrowBodyWidth/2,y1-ArrowHeadLength );
	TheArrowShape.lineTo( x1-ArrowBodyWidth/2,y1-ArrowHeadLength-ArrowBodyLength );
	TheArrowShape.lineTo( x1+ArrowBodyWidth/2,y1-ArrowHeadLength-ArrowBodyLength );
	TheArrowShape.lineTo(  x1+ArrowBodyWidth/2,y1-ArrowHeadLength );
	TheArrowShape.lineTo( x1+ArrowHeadWidth,y1-ArrowHeadLength-ArrowBarbLength );
	TheArrowShape.lineTo( x1,y1 );
	
	var extrudeSettings = { depth: 8, bevelEnabled: true, bevelSegments: 2, steps: 2, bevelSize: 1, bevelThickness: 1 };
	
	var geometry = new THREE.ExtrudeGeometry( TheArrowShape, extrudeSettings );

	var SolidColorMaterial=new THREE.MeshPhongMaterial({
		  color: 0x0000FF,
		  side:THREE.FrontSide, 
		  opacity:1,
		  transparent :false
	});	
	
	TheArrow = new THREE.Mesh(geometry, SolidColorMaterial);

	//
	
	TheArrow.Type=CM3LayerSpecial.ARROW;

	var TheScene=this.GetParent(CM3Scene);
		
	TheScene.GetGeo().AddOGLObjectToGeo(TheArrow);

	this.TheObjects.push(TheArrow);
}

CM3LayerSpecial.prototype.AddObjFile=function()
{
	var URL="GISData/cylinder.obj";
	
	var TheRequest=new XMLHttpRequest(); // wait staff at the resturant
	TheRequest.open("GET",URL,true); // the URL is what we ordered
	TheRequest.TheURL=URL;
	TheRequest.TheDataset=this;
			
	TheRequest.onreadystatechange=function() 
	{
		if (this.readyState == 4)  // done
		{
			if (this.status == 200) // OK
			{
				var TheText=TheRequest.responseText;

				var Lines=TheText.split("\n");
				
				// get the vertices
				
				var Vertices=[];
				var NumVertices=0;
				
				var TheGeometry = new THREE.Geometry();
				for (var i=0;i<Lines.length;i++)
				{
					var Tokens=Lines[i].split(" ");
					if (Tokens[0]=="v") // vertex
					{
						var Coordinate=[];
						
						Coordinate[0]=parseFloat(Tokens[1])*100;
						Coordinate[1]=parseFloat(Tokens[2])*100;
						Coordinate[2]=parseFloat(Tokens[3])*100;
						
						Vertices.push(Coordinate);
							TheGeometry.vertices.push(new THREE.Vector3( Coordinate[0],Coordinate[1],Coordinate[2] ));
						NumVertices++;
					}
				}
				
				// setup to add the geometries
				
				var TheScene=this.TheDataset.GetParent(CM3Scene);
				
				var TheColor=CMUtilities.GetRGBAFromAnyColor("rgb(233,0,0)");
				TheColor=new THREE.Color(TheColor);
				
				var SolidColorMaterial=new THREE.MeshPhongMaterial({
					  color: TheColor,
					  side:THREE.DoubleSide, 
					  opacity:1,
					  transparent :false
				});	
				
				// get the faces
				
				var NumFaces=0;
				var Done=false;
				for (var i=0;(i<Lines.length)&&(Done==false);i++)
				{
					var Tokens=Lines[i].split(" ");
					if (Tokens[0]=="f") // face
					{
						// get the vertex indexes for the faces
						var VertexIndexes=[];
						
						for (var j=1;j<Tokens.length;j++)
						{
							var VertexTokens=Tokens[j];
							
							var VertexInfoTokens=VertexTokens.split("/");
							
							var Index=parseInt(VertexInfoTokens[0])-1;
							
							VertexIndexes.push(Index);
						}
						
						if (VertexIndexes.length==3)
						{
							TheGeometry.faces.push( new THREE.Face3( VertexIndexes[0],VertexIndexes[1],VertexIndexes[2] ) ); 
							NumFaces++;
						}
						else if (VertexIndexes.length==4)
						{
							TheGeometry.faces.push( new THREE.Face4( VertexIndexes[0],VertexIndexes[1],
								VertexIndexes[2],VertexIndexes[3]) ); 
							NumFaces++;
						}
						else // ngon, we have to create a 2D shape and then add it immediately (should combine into larger shape)
						{
							var ThePoints = [];

							for (var j=0;j<VertexIndexes.length;j++)
							{
								var TheIndex=VertexIndexes[j];
								
								var TheCoordinate=Vertices[TheIndex];
								
								ThePoints.push( new THREE.Vector2 ( TheCoordinate[0], TheCoordinate[2] ) );
							}
							
							var TheShape = new THREE.Shape( ThePoints );

							var geometry = new THREE.ShapeGeometry( TheShape );
							
							geometry.computeBoundingSphere();
							geometry.computeFaceNormals ()
							this.TheOGLObject = new THREE.Mesh(geometry, SolidColorMaterial);
							TheScene.GetOGLScene().add(this.TheOGLObject);
							this.TheOGLObject.TheCM3Item=this;
							
							NumFaces++;
						}
						//Done=true;
					}
				}
				TheGeometry.computeBoundingSphere();
				TheGeometry.computeFaceNormals ()
				
				//
				
				this.TheOGLObject = new THREE.Mesh(TheGeometry, SolidColorMaterial);
				TheScene.GetOGLScene().add(this.TheOGLObject);
				this.TheOGLObject.TheCM3Item=this;
			
				//
				
				var ThBounds={
					XMin:-10,
					XMax:10,
					YMin:-10,
					YMax:10
				}
				TheScene.GetGeo().ZoomToBounds(ThBounds);
			}
			else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
		}
	}
	TheRequest.send();
}

//**************************************************************************************************
// Public CMItem Functions
//**************************************************************************************************

//CM3LayerSpecial.prototype.CMLayer_SetOGLObjectVisible=CMLayer.prototype.SetOGLObjectVisible;

CM3LayerSpecial.prototype.SetOGLObjectVisible=function(Flag)
{
	if (this.GetVisible()!=Flag)
	{
		//this.CMLayer_SetOGLObjectVisible(Flag);
		
		var TheScene=this.GetParent(CM3Scene);
		
		if (Flag)  // add the mesh to the TheOGLScene to make it visible
		{
			for (var i=0;i<this.TheObjects.length;i++)
			{
				TheScene.AddSelectableObject(this.TheObjects[i]);
			
				if (this.TheObjects[i].Type==CM3LayerSpecial.FLOOR)
				{
					TheScene.AddOGLObject(this.TheObjects[i]);
				}
				else
				{
					TheScene.GetGeo().AddOGLObjectToGeo(this.TheObjects[i]);
				}
			}
		}
		else // remove the meshfrom the TheOGLScene to make it invisible
		{
			for (var i=0;i<this.TheObjects.length;i++)
			{
				TheScene.RemoveSelectableObject(this.TheObjects[i]);
			
				if (this.TheObjects[i].Type==CM3LayerSpecial.FLOOR)
				{
					TheScene.RemoveOGLObject(this.TheObjects[i]);
				}
				else
				{
					TheScene.GetGeo().RemoveOGLObjectFromGeo(this.TheObjects[i]);
				}
			}
		}
	}
}
//**************************************************************************************************
// Public CMLayer Functions
//**************************************************************************************************
CM3LayerSpecial.prototype.AddItemToPopup=function(ThePopupMenu,TheText,Type)
{
	var PropertiesElement=document.createElement('div');
	PropertiesElement.setAttribute("id","CM_SettingsElementMenuItem");
	PropertiesElement.className="CM_LayerListPopupMenuItem";
	PropertiesElement.innerHTML=TheText;
	
	PropertiesElement.TheLayer=this;
	PropertiesElement.ThePopupMenu=ThePopupMenu;
	PropertiesElement.Type=Type;
	
	PropertiesElement.addEventListener('click', function(event)
	{
		this.ThePopupMenu.style.visibility= "hidden";
		
		if (Type==CM3LayerSpecial.FLOOR)
		{	
			this.TheLayer.AddFloor();
		}
		else if (Type==CM3LayerSpecial.COMPASS_ROSE)
		{	
			this.TheLayer.AddCompassRose();
		}
		else if (Type==CM3LayerSpecial.SPHERICAL_EARTH)
		{	
			this.TheLayer.AddSphericalEarth();
		}
		else if (Type==CM3LayerSpecial.ARROW)
		{	
			this.TheLayer.AddArrow();
		}
		else if (Type==CM3LayerSpecial.OBJ_FILE)
		{	
			this.TheLayer.AddObjFile();
		}
		event.stopPropagation();
	});
	ThePopupMenu.appendChild(PropertiesElement);
}

CM3LayerSpecial.prototype.CMLayer_FillPopupMenu=CMLayer.prototype.FillPopupMenu;

CM3LayerSpecial.prototype.FillPopupMenu=function(ThePopupMenu)
{
	this.CMLayer_FillPopupMenu(ThePopupMenu);
	
	this.AddItemToPopup(ThePopupMenu,"Add Floor To Scene...",CM3LayerSpecial.FLOOR);
	
	this.AddItemToPopup(ThePopupMenu,"Add Compass Rose...",CM3LayerSpecial.COMPASS_ROSE);
	
//	this.AddItemToPopup(ThePopupMenu,"Add Spherical Earth To Scene...",CM3LayerSpecial.SPHERICAL_EARTH);
	
	this.AddItemToPopup(ThePopupMenu,"Add Arrow...",CM3LayerSpecial.ARROW);
	
	this.AddItemToPopup(ThePopupMenu,"Add Obj File...",CM3LayerSpecial.OBJ_FILE);

}
