/**************************************************************************************************
* CM3LayerVector Class
*
* @module CM3LayerVector
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
/*
* Constructor
* @public, @constructs
*/
function CM3LayerVector()
{
	CMLayerDataset.call(this);

	// settings
	
	this.TimeSlices[0].Settings.Elevation=	
	{
	};
	this.TimeSlices[0].Settings.Extrusion=	
	{
		bevelEnabled:true,
		bevelSegments: 5,
		bevelSize:0.2,
		bevelThickness:0.2,
	};
	this.TimeSlices[0].Settings.CrossSection=	
	{
	};
	
	this.TheMaterial=null;
	this.TheFeatureMaterials=null;
	
	this.TheTerrainDataset=null;
	
	// other properties
	this.TheFeatureMeshes=null;
	this.SettingsDirty=true;
	
	this.Anchor=new THREE.Vector3();
	this.DraggingFeatureIndex=-1;
	this.DraggingPlane=new THREE.Plane(); // Plane that objects are dragged within (normal plane to the camera at the item intersected with mouse?
	this.IntersectionPoint=new THREE.Vector3();
};

CM3LayerVector.prototype=Object.create(CMLayerDataset.prototype); // inherit prototype functions from PanelBase()

CM3LayerVector.prototype.contructor=CM3LayerVector; // override the constructor to go to ours
//**************************************************************************************************
// Setting Definitions
//**************************************************************************************************

CM3LayerVector.SettingDefintions=
{
	Elevation:
	{
		ZOffset: { Name:"ZOffset",Type:CMBase.DATA_TYPE_FLOAT, Default:0 },
		ZMultiplier: { Name:"ZMultiplier",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
	},
	// settings for the Three.js extrusion
	Extrusion:
	{
		depth: { Name:"Depth",Type:CMBase.DATA_TYPE_FLOAT, Default:10 }, // amount to extrude the object
		steps: { Name:"Steps",Type:CMBase.DATA_TYPE_INTEGER, Default:2 },
		bevelEnabled: { Name:"bevelEnabled",Type:CMBase.DATA_TYPE_BOOLEAN, Default:true },
		bevelSegments: { Name:"bevelSegments",Type:CMBase.DATA_TYPE_FLOAT, Default:5 },
		bevelSize: { Name:"bevelSize",Type:CMBase.DATA_TYPE_FLOAT, Default:0.2 },
		bevelThickness: { Name:"bevelThickness",Type:CMBase.DATA_TYPE_FLOAT, Default:0.2 },
	},
	CrossSection:
	{
		Radius: { Name:"Radius",Type:CMBase.DATA_TYPE_FLOAT, Default:100 }, 
		NumFaces: { Name:"NumFaces",Type:CMBase.DATA_TYPE_INTEGER, Default:2 },
	},
}
//**************************************************************************************************
// Private static debugging functions
//**************************************************************************************************
/**
* Adds a point of the specified color into the OGL scene
*/
CM3LayerVector.AddPointToScene=function(TheOGLScene,RotationPoint,TheColor,Radius)
{
	if (RotationPoint!=undefined)
	{
		if (Radius==undefined) Radius=1;
		
		var geometry = new THREE.SphereGeometry( Radius, 8, 8 );
		geometry.translate(RotationPoint.x,RotationPoint.y,RotationPoint.z);
				
		var material = new THREE.MeshBasicMaterial( {color: TheColor} );
		var sphere = new THREE.Mesh( geometry, material );
		TheOGLScene.add( sphere );
	}
}

//**************************************************************************************************
// Private static functions for converting GeoJSON to 3d geometries
//**************************************************************************************************

//*******************************************************************
/**
 * Converst a GeoJSON set of coordinates into a THREE shape.
 * @param RefX - coordinate x value in units compatible with the Geo (typically lon)
 * @param RefY
 * @private
 */
CM3LayerVector.prototype.Get3DShapeFromCoordinate=function(RefX,RefY,Distance)
{
	var ThePoints = [];
	
	var TheGeo=this.GetParent(CM3Geo);

	//RefX and 
	
	var Result=TheGeo.ProjectFromGeographic(RefX,RefY,0);
	
	RefX=Result[0];
	RefY=Result[1];
	RefZ=Result[2];
	
	ThePoints.push( new THREE.Vector3 ( RefX+Distance, RefY+Distance,RefZ ) );
	ThePoints.push( new THREE.Vector3 ( RefX-Distance, RefY+Distance,RefZ ) );
	ThePoints.push( new THREE.Vector3 ( RefX-Distance, RefY-Distance,RefZ ) );
	ThePoints.push( new THREE.Vector3 ( RefX+Distance, RefY-Distance,RefZ ) );
	
	var TheShape = new THREE.Shape( ThePoints );

	return(TheShape);
}
/**
 * Converst a GeoJSON set of coordinates into a THREE shape.
 * @private
 */
CM3LayerVector.prototype.Get3DShapeFromCoordinates=function(Xs,Ys)
{
	var ThePoints = [];

	for (var j=0; j < Xs.length; j++) 
	{
		RefX=Xs[j];
		RefY=Ys[j];

		ThePoints.push( new THREE.Vector2 ( RefX, RefY ) );
	}
	var TheShape = new THREE.Shape( ThePoints );

	return(TheShape);
}
CM3LayerVector.prototype.Get3DShapesFromRegion=function(TheRegion,TheType)
{
	var TheShapes=[]; // 3d Shapes for this feature (could be multiple)
	
	// get the geometry so we can see if it is a point feature
	
	for (var AreaIndex=0;AreaIndex<TheRegion.length;AreaIndex++)
	{
		var TheArea=TheRegion[AreaIndex];
		
		var ExteriorPoly=TheArea[0];
		
		switch (TheType)
		{
		case CMDatasetVector.TYPE_POINTS:
			var Size=this.GetSetting("Mark","Size");
			var TheShape=this.Get3DShapeFromCoordinate(ExteriorPoly[0][0],ExteriorPoly[1][0],Size*200000); 
			TheShapes.push(TheShape);
			break;
		default:
			var TheShape=this.Get3DShapeFromCoordinates(ExteriorPoly[0],ExteriorPoly[1]);
			TheShapes.push(TheShape);
			break;
		}
	}
	return(TheShapes);
}
CM3LayerVector.prototype.Get3DShapesFromRegions=function(TheRegions,TheType)
{
	var FeatureShapes=[];
	
	if (TheRegions!=null)
	{
		// draw each feature
		for (var RegionIndex=0; RegionIndex < TheRegions.length; RegionIndex++) 
		{
			var TheRegion=TheRegions[RegionIndex];
			
			var TheShapes=Get3DShapesFromRegion(TheRegion,TheType);
			
			FeatureShapes[RegionIndex]=TheShapes;
		}
	}
	return(FeatureShapes);
}
//***********************************************************************
//  Private CM3LayerVector functions
//***********************************************************************
/**
* For spherical projections, break up polygons into cells (does not work well yet)
*/
CM3LayerVector.GreatArcDistanceAlongMedian=function(SouthLat,NorthLat,Radius)
{
	var SouthLatInRadians=(SouthLat)/180*Math.PI;
	var NorthLatInRadians=(NorthLat)/180*Math.PI;
	
	var Distance=(NorthLatInRadians-SouthLatInRadians)*Radius;
	
	return(Distance);
}
CM3LayerVector.GreatArcDistanceAlongParallel=function(WestLongitude,EastLongitude,Lat2,Radius)
{
	if (Lat2<0) Lat2=-Lat2;
	
	var LatInRadians=Lat2/180*Math.PI;
	
	var ProportionRadiusAtParallel=Math.cos(LatInRadians);
	
	var RadiusAtParallel=ProportionRadiusAtParallel*Radius;
	
	var ParallelDistance=RadiusAtParallel*Math.PI*2;
	
	var DeltaLon=(EastLongitude-WestLongitude)/180*Math.PI;
	
	var Factor=DeltaLon/(Math.PI*2);
	
	var Distance=ParallelDistance*Factor;
	
	return(Distance);
}
/*
* Creates a feature mesh based on the data set and current settings.
* @private
*/

CM3LayerVector.GetExtrudedPointMesh=function(Size,extrudeSettings,EarthMaterial)
{
	var ThePoints = [];
	
	ThePoints.push( new THREE.Vector3 ( Size, Size,0 ) );
	ThePoints.push( new THREE.Vector3 ( -Size,Size,0 ) );
	ThePoints.push( new THREE.Vector3 ( -Size, -Size,0 ) );
	ThePoints.push( new THREE.Vector3 ( Size, -Size,0 ) );
	
	var TheShape = new THREE.Shape( ThePoints );
	
	var geometry = new THREE.ExtrudeGeometry( TheShape, extrudeSettings );
	
	// add the geometry to the mesh
	var mesh1 = new THREE.Mesh( geometry,EarthMaterial);
	
	return(mesh1);
}
/*
* Creates a feature mesh based on the data set and current settings.
* @private
*/

CM3LayerVector.prototype.GetFeatureMesh=function(FeatureIndex)
{
	var TheFeatureMesh=null; 
	var Result=null;// return value
	
	var RegionIndex=FeatureIndex;
	
	var TheDataSet=this.GetDataset();
	
	var TheRegions=TheDataSet.GetRegions();
	var TheType=TheDataSet.GetType();
	
	var TheGeo=this.GetParent(CM3Geo);

	var TheProjector=TheGeo.GetProjector();
	
	var TheRegion=TheRegions[FeatureIndex];
	
	var TheShapes=[]; // 3d Shapes for this feature (could be multiple)
	
	var TheScene=this.GetParent(CM3Scene);
	var TheOGLScene = TheScene.GetOGLScene()
	
	// get the geometry so we can see if it is a point feature
	
	if (TheRegion.length>1) Result=new THREE.Group();
	
	for (var AreaIndex=0;AreaIndex<TheRegion.length;AreaIndex++)
	{
		var TheArea=TheRegion[AreaIndex];
		
		var ExteriorPoly=TheArea[0];
		
		// get the size of the shape
		var Size=this.GetSetting("Mark","Size");
		var ZOffset=this.GetSetting("Elevation","ZOffset");
		var ZMultiplier=this.GetSetting("Elevation","ZMultiplier",1);

		// extrude settings
		
		var ExtrudeSettings=this.GetSettingGroup("Extrusion");
		
		var depth=this.GetFeatureSetting("Extrusion","depth",RegionIndex,10000);
		var steps=this.GetFeatureSetting("Extrusion","steps",RegionIndex,10);
		var bevelEnabled=this.GetFeatureSetting("Extrusion","bevelEnabled",RegionIndex,true);
	
		var extrudeSettings = { 
			depth: depth,  // distance of extrusion
			steps: steps,  // points use to subdivide along the spline
			bevelEnabled: bevelEnabled,  // beveled instead of 90 degree change
			bevelSegments: ExtrudeSettings.bevelSegments,  // number of bevel layers (more makes a smoother bevel but draws slower
			bevelSize: ExtrudeSettings.bevelSize,  // width of bevel (distance from shape outline
			bevelThickness: ExtrudeSettings.bevelThickness // how deep the bevel goes into the extrusion 
//						extrudePath  // THREE.CurvePath for extrusion to follow (creates frames if not defined)
		 };

		//*******************************************
		// create the materials

		var EarthMaterial=null;
		
		if (this.TheMaterial!=null)
		{
			EarthMaterial=this.TheMaterial;
		}
		if (this.TheFeatureMaterials!=null)
		{
			if (this.TheFeatureMaterials[RegionIndex]!=null)
			{
				EarthMaterial=this.TheFeatureMaterials[RegionIndex];
			}
		}
		
		//*******************************************
		// create the geometry, add it to a mesh, and insert it into the FeatureMesh array based on 
		// the type of vector data
		switch (TheType)
		{
		case CMDatasetVector.TYPE_POINTS:
			if (EarthMaterial==null)
			{
				var TheColor=this.GetFeatureSetting("Style","fillStyle",RegionIndex,"rgb(255,255,255)");
				
				TheColor=CM3Utilities.Get3DColorFromCanvasColor(TheColor);
				
				EarthMaterial=new THREE.MeshPhongMaterial( { 
					color:TheColor.RGB, //Result.RGB
					opacity:TheColor.Opacity,
					transparent:TheColor.Transparent
				} ) ;
			}
			// create the top shape
			var mesh1=CM3LayerVector.GetExtrudedPointMesh(Size,extrudeSettings,EarthMaterial);
			
			// get the coordinate
			var RefX=ExteriorPoly[0][0];
			var RefY=ExteriorPoly[1][0];
			var RefZ=0;
			
			// find a z value if available
			
			if (ExteriorPoly.length>2) // get the z value from the points
			{
				RefZ=ExteriorPoly[2][0]*ZMultiplier+ZOffset;
			}
			else if (this.TheTerrainDataset!=null) // see if there is a DEM for a z value
			{
				var DEM_ZOffset=this.TheTerrainDataset.GetSampleFromRef(RefX,RefY);
				
				if (DEM_ZOffset!=undefined) 
				{
					RefZ+=DEM_ZOffset*ZMultiplier+ZOffset;
				}
			}
			else
			{
				RefZ=ZOffset;
			}
			//***********************************************************
			// Setup the appropriate group to add the points to the geo
			
			var group2 = null;
			
			if (TheProjector==null)
			{
				mesh1.position.set(RefX,RefY,RefZ); //6371000
			
				group2=new THREE.Group();
				group2.add( mesh1 );
			}
			else if (TheProjector instanceof CM3ProjectorSpherical)
			{
				var Lon=RefX;
				var Lat=RefY;
				
				group2=TheProjector.GetPositionedMesh(mesh1,Lon,Lat,ZOffset+RefZ);
			}
			else
			{
				//var Vector=TheGeo.ProjectFromGeographic(Lon,Lat,0);
				
			}
			// save the mesh
			TheFeatureMesh=group2;
		
			break;
			
		//************************************************************************************
		case CMDatasetVector.TYPE_POLYLINES:
			ExteriorPoly=TheArea[0];
			
			var group1 = new THREE.Group();
			
			// get the coordinates
			var Xs=ExteriorPoly[0];
			var Ys=ExteriorPoly[1];
			
			var Zs=null;
			if (ExteriorPoly.length>2) Zs=ExteriorPoly[2]; // get the z value from the points
			
			//********************************************
			// Convert the polyline to a path array (array of THREE.Vector3s)
			
			var PathArray=[];
			
			if (this.TheTerrainDataset==null) // do not have terrain
			{
				if ((typeof(CM3ProjectorSpherical)!="undefined") && (TheProjector instanceof CM3ProjectorSpherical))
				{
					for (var j=0;j<Xs.length;j++)
					{
						var RefX=Xs[j];
						var RefY=Ys[j];
						var RefZ=0;
						
						if (Zs!=null) RefZ=Zs[j];// get the z value from the points

						var Lon=RefX;
						var Lat=RefY;
						
						// 
						var LonInRadians=-Lon/180*Math.PI+Math.PI/2;
						var LatInRadians=Lat/180*Math.PI;
						
						var Radius=CM3ProjectorSpherical.Radius+ZOffset+RefZ;
						
						var Z=Math.sin(LatInRadians)*Radius;
						
						var Y=Math.cos(LonInRadians)*Math.cos(LatInRadians)*Radius;
						var X=Math.sin(LonInRadians)*Math.cos(LatInRadians)*Radius;
						
						PathArray.push(new THREE.Vector3(X,Y,Z));
					}
				}
				else  // do not have a spherical projector
				{
					for (var j=0;j<Xs.length;j++)
					{
						var RefX=Xs[j];
						var RefY=Ys[j];
						var RefZ=0;
						
						if (Zs!=null) RefZ=Zs[j];// get the z value from the points
						
						// apply the mulitplier and offset
						RefZ=RefZ*ZMultiplier+ZOffset;
						
						PathArray.push(new THREE.Vector3(RefX,RefY,RefZ));
					}
					if (false)
					{
						for (var PathArrayIndex=0;PathArrayIndex<PathArray.length;PathArrayIndex++)
						{
							// debugging
							CM3LayerVector.AddPointToScene(TheOGLScene,PathArray[PathArrayIndex],0x0000ff,0.1);
						}
					}
				}
			}
			else // have a terrain dataset, have the polyline follow the terrain
			{
				PathArray=this.TheTerrainDataset.GetPathArray(Xs,Ys,Zs,ZOffset,ZMultiplier);
				
				if (false)
				{
					for (var PathArrayIndex=0;PathArrayIndex<PathArray.length;PathArrayIndex++)
					{
						// debugging
						CM3LayerVector.AddPointToScene(TheOGLScene,PathArray[PathArrayIndex],0x0000ff);
					}
				}
			}
			
			//********************************************
			// get the geometry
			
			// get the extrusion
			var ThePathExtrusion=this.GetPathExtrusion();
			
			if (ThePathExtrusion!=undefined) // extrusion
			{
				if (EarthMaterial==null)
				{
					var TheColor=this.GetFeatureSetting("Style","fillStyle",RegionIndex,"rgb(255,255,255)");
					
					TheColor=CM3Utilities.Get3DColorFromCanvasColor(TheColor);
					
					EarthMaterial=new THREE.MeshPhongMaterial( { 
						color:TheColor.RGB, //Result.RGB
						opacity:TheColor.Opacity,
						transparent:TheColor.Transparent
					} ) ;
				}
			
				// get the cross section array
				var CrossSectionArray=ThePathExtrusion.GetCrossSectionArray();
				
				if (CrossSectionArray==undefined)
				{
					var NumFaces=this.GetSetting("CrossSection","NumFaces",2);
					var Radius=this.GetSetting("CrossSection","Radius",1);
			
					// Get the cross section for the extrusion along the polyline
					CrossSectionArray=CM3Utilities.GetArcArrayInDegrees(0,360,NumFaces,Radius); // Get a simple cross section from 0 to 360 degrees with 2 segments and a radius of 0.2
				}
				// estup the path extrusion to create the geometry
				ThePathExtrusion.SetCrossSectionArray(CrossSectionArray); // set the cross section into the extrusion
				
				ThePathExtrusion.SetPathArray(PathArray);
	
				var TheGeometry=ThePathExtrusion.GetGeometry();
				
				//********************************************
				// add the geometry to the group
				var mesh1 = new THREE.Mesh( TheGeometry,EarthMaterial);
					
				group1.add(mesh1);
			}
			else // no CMPathExtrusion
			{
				if (EarthMaterial==null)
				{
					var TheColor=this.GetFeatureSetting("Style","fillStyle",RegionIndex,"rgb(255,255,255)");
					
					TheColor=CM3Utilities.Get3DColorFromCanvasColor(TheColor);
					
					EarthMaterial=new THREE.LineBasicMaterial( { 
						color:TheColor.RGB, 
					} ) ;
				}

				//var TheShape=new THREE.Shape(PathArray);
				var TheGeometry = new THREE.BufferGeometry().setFromPoints( PathArray );
				
				var mesh1 = new THREE.Line( TheGeometry,EarthMaterial  );
					
				group1.add(mesh1);
			}
			// add the geometry to the array of features 
			TheFeatureMesh=group1;
			break;
		//**************************************************************************************
		case CMDatasetVector.TYPE_POLYGONS:
			if (EarthMaterial==null)
			{
				var TheColor=this.GetFeatureSetting("Style","fillStyle",RegionIndex,"rgb(255,255,255)");
				
				TheColor=CM3Utilities.Get3DColorFromCanvasColor(TheColor);
				
				EarthMaterial=new THREE.MeshPhongMaterial( { 
					color:TheColor.RGB, //Result.RGB
					opacity:TheColor.Opacity,
					transparent:TheColor.Transparent
				} ) ;
			}
			
			if (TheProjector==null)
			{
				var TheShape=this.Get3DShapeFromCoordinates(ExteriorPoly[0],ExteriorPoly[1]);
				
				var geometry = new THREE.ExtrudeGeometry( TheShape, extrudeSettings );
					
				// add the geometry to the mesh
				var mesh1 = new THREE.Mesh( geometry,EarthMaterial);
				
				TheFeatureMesh=mesh1;
//				this.TheFeatureMeshes.push(mesh1);
			}
			else if (TheProjector instanceof CM3ProjectorSpherical)
			{
				var group3 = new THREE.Group();
				
				var CellWidth=5;
				var NumRows=180/CellWidth;
				var NumColumns=360/CellWidth;
				
				// go through the lat lon grid
				for (var Row=0;Row<NumRows;Row++)
				{
					for (var Column=0;Column<NumColumns;Column++)
					{
						var LonLeft=Column*CellWidth-180;
						var LatLower=Row*CellWidth-90;
						
						// center
						var Lon=LonLeft+CellWidth/2;
						var Lat=LatLower+CellWidth/2;
						
						var ClippingBox=[LonLeft,LatLower,LonLeft+CellWidth,LatLower+CellWidth];
		
						var ClippedArea=CMUtilities.ClipArea(TheArea,ClippingBox,CMDatasetVector.TYPE_POLYGONS);
						
						if (ClippedArea.length>0)
						{
							var ClippedPoly=ClippedArea[0];
							
							// move the poly to be referened in the cell
							
							var Xs=ClippedPoly[0];
							var Ys=ClippedPoly[1];
							
							for (var CoordinateIndex=0;CoordinateIndex<Xs.length;CoordinateIndex++)
							{
								var XDistance=CM3LayerVector.GreatArcDistanceAlongParallel(LonLeft,Xs[CoordinateIndex],Ys[CoordinateIndex],CM3ProjectorSpherical.Radius);
								var YDistance=CM3LayerVector.GreatArcDistanceAlongMedian(LatLower,Ys[CoordinateIndex],CM3ProjectorSpherical.Radius);
								
								Xs[CoordinateIndex]=XDistance;
								Ys[CoordinateIndex]=YDistance;
							}
							var TheShape=this.Get3DShapeFromCoordinates(Xs,Ys);
							
							// create the geometry
							var geometry = new THREE.ExtrudeGeometry( TheShape, extrudeSettings );
								
							// add the geometry to the mesh
							var mesh1 = new THREE.Mesh( geometry,EarthMaterial);

							// setup the two groups with the rotations required for lat/lon
							
							var LonInRadians=LonLeft/180*Math.PI;
							var LatInRadians=LatLower/180*Math.PI;
							
							// add a group and rotate it for latitude
							var group1 = new THREE.Group();
							group1.rotation.set(0,LatInRadians-Math.PI/2,3*Math.PI/2);
							
							// add a group and rotate it for longitude
							var group2 = new THREE.Group();
							group2.add(group1);
							group2.rotation.set(0,0,LonInRadians-Math.PI);
							
							// move the shape to the elevation
							mesh1.position.set( 0,0,CM3ProjectorSpherical.Radius+ZOffset); //
						
							// add the shape 
							group1.add( mesh1 );
							
							group3.add(group2);
						}
					}
				}
				//this.TheFeatureMeshes.push(group3);
				TheFeatureMesh=group3;
			}
			break;
		}
		if (TheRegion.length>1) 
		{
			Result.add(TheFeatureMesh);
		}
		else Result=TheFeatureMesh;
		
		var PositionOffset=this.GetSetting("Item","PositionOffset");
		if (PositionOffset!=null) 
		{
			Result.translateX(PositionOffset[0]);
			Result.translateY(PositionOffset[1]);
			Result.translateZ(PositionOffset[2]);
		}
	}
	return(Result);
}
/**
* Called to create the feature meshes
* @private
*/
CM3LayerVector.prototype.GetFeatureMeshes=function()
{
	if ((this.TheFeatureMeshes==null)||(this.SettingsDirty))
	{
		//var Radius=CM3ProjectorSpherical.Radius+100;
		var TheDataSet=this.GetDataset();
		var TheGeo=this.GetParent(CM3Geo);
		
		if ((CMUtilities.IsDefined(TheDataSet))&&(TheDataSet.GetRegions()!=null)&&(true))
		{
			// if the settings were dirty and the meshes were not removed from the OGL, do it now
/*			if (this.TheFeatureMeshes!=null)
			{
				this.UpdateMeshInGeo(false); // remove the meshes from the geo
				this.TheFeatureMeshes=null;
			}
*/			this.TheFeatureMeshes=[];
		
			var TheRegions=TheDataSet.GetRegions();
			var TheType=TheDataSet.GetType();
			
			var TheGeo=this.GetParent(CM3Geo);
		
			var TheProjector=TheGeo.GetProjector();
			
			for (var RegionIndex=0;RegionIndex < TheRegions.length; RegionIndex++) 
			{
				var TheFeatureMesh=this.GetFeatureMesh(RegionIndex);
				
				this.TheFeatureMeshes[RegionIndex]=TheFeatureMesh;
			}
			this.SettingsDirty=false;
		}
		
		//this.SetVisible(true);
	}
	return(this.TheFeatureMeshes);
}
/**
* Either adds or removes the meshes in the geo's OGL object based on the Visible parameter.
* @private
*/
CM3LayerVector.prototype.UpdateMeshInGeo=function(Visible)
{
	//var TheFeatureMeshes=this.GetFeatureMeshes();
	
	if (this.TheFeatureMeshes!=null)
	{
		var TheScene=this.GetParent(CM3Scene);
		
		var TheGeo=TheScene.GetGeo();
		
		if (Visible==undefined) Visible=this.GetVisible();
		
		for (var FeatureIndex=0;FeatureIndex<this.TheFeatureMeshes.length;FeatureIndex++)
		{
			var TheMesh=this.TheFeatureMeshes[FeatureIndex];
			
			if (TheMesh!=null)
			{
				if (Visible)  // add the mesh to the TheOGLScene to make it visible
				{
					TheGeo.AddOGLObjectToGeo(TheMesh);
				}
				else // remove the meshfrom the TheOGLScene to make it invisible
				{
					TheGeo.RemoveOGLObjectFromGeo(TheMesh);
				}
			}
		}
	}
}
//**************************************************************************************************
// CMBase Functions
//**************************************************************************************************
CM3LayerVector.prototype.CMLayerDataset_GetSettingsDefinitions=CMLayerDataset.prototype.GetSettingsDefinitions; // save the superclass function

CM3LayerVector.prototype.GetSettingsDefinitions=function() 
{
	var Result=this.CMLayerDataset_GetSettingsDefinitions(); // get the settings definitions from the superclass
	
	for (Key in CM3LayerVector.SettingDefintions)
	{
		Result[Key]=CM3LayerVector.SettingDefintions[Key];
	}
	return(Result); 
}

CM3LayerVector.prototype.CMItem_SetSettings=CMItem.prototype.SetSettings;

CM3LayerVector.prototype.SetSettings=function(NewSettings,TimeSlice) 
{
	this.CMItem_SetSettings(NewSettings,TimeSlice);
	
	this.UpdateMeshInGeo(false); // remove the meshes from the geo
	
	this.SettingsDirty=true; // set that an update is needed
}
/**
* Utility function to call old event handlers
* @public
*/
CM3LayerVector.prototype.SendMessageToDescendants=function(Message,AdditionalInfo)
{
	Used=false;
	
	switch (Message)
	{
	case CMView.MESSAGE_MOUSE_DOWN:
		Used=this.MouseDown(AdditionalInfo.TheView,AdditionalInfo.TheEvent,AdditionalInfo.TheRayTracer);
		break;
	case CMView.MESSAGE_MOUSE_MOVE:
		Used=this.MouseMove(AdditionalInfo.TheView,AdditionalInfo.TheEvent,AdditionalInfo.TheRayTracer);
		break;
	case CMView.MESSAGE_MOUSE_UP:
		Used=this.MouseUp(AdditionalInfo.TheView,AdditionalInfo.TheEvent,AdditionalInfo.TheRayTracer);
		break;
	}
	return(Used);
}

//***********************************************************************
// CMItem functions
//***********************************************************************

CM3LayerVector.prototype.CMLayerDataset_SetVisible=CMLayerDataset.prototype.SetVisible;

CM3LayerVector.prototype.SetVisible=function(Flag)
{
	if (this.GetVisible()!=Flag) // visibility has changed
	{
		this.CMLayerDataset_SetVisible(Flag); // need to inform children that the layer visibility has changed
		
		if (Flag) this.GetFeatureMeshes(); // meshes becoming visible
		
		this.UpdateMeshInGeo(Flag);
	}
}

/* Paint really means that something has changed (or mouse moved, change that later)
* 3D envrionment is updated in SetVisible()
*/
CM3LayerVector.prototype.Paint=function(TheView) 
{
	if (this.SettingsDirty)
	{
		this.GetFeatureMeshes();
		
		this.UpdateMeshInGeo();
	}
}
/**
* Finds if the mouse was pressed down in a feature in the layer
* @public
* @param - TheView
* @param - TheEvent
* @param - TheRayTracer
* @returns Result - { FeatureIndex,MeshIndex } or the feature and mesh clicked in, null otherwise
*/
CM3LayerVector.prototype.In=function(TheView,TheEvent,TheRayTracer)
{
	var Result=null;
	
	if (this.TheFeatureMeshes!=null)
	{
		for (var FeatureIndex=0;(FeatureIndex<this.TheFeatureMeshes.length)&&(Result==null);FeatureIndex++)
		{
			var TheMesh=this.TheFeatureMeshes[FeatureIndex];
			
			for (var j=0;j<TheMesh.children.length;j++)
			{
				var TheChild=TheMesh.children[j];
				
				var SelectedObjects=TheRayTracer.intersectObject(TheChild,false); // find the objects the ray intersects with
				
				if (SelectedObjects.length>0)
				{
					if (TheRayTracer.ray.intersectPlane(this.DraggingPlane, this.IntersectionPoint)) 
					{
						Result={ FeatureIndex:FeatureIndex,MeshIndex:j};
						
						// Find the difference between the intersection with the object and the object's origin and save as the anchor point
						this.Anchor.copy( this.IntersectionPoint ).sub( TheChild.position ); 
					}
				}
			}
		}
	}
	return(Result);
}
/**
* This mouse down is different from 2D as we know this item had the mouse clicked down in it.
* @public
*/
CM3LayerVector.prototype.MouseDown=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
	
	if (this.GetSetting("Item","Status")==CMItem.STATUS_EDITABLE)
	{
		TheView.SetCursor('move'); // change the cursor to move the object
		
		var Result=this.In(TheView,TheEvent,TheRayTracer);
		
		if (Result!=null) // mouse down in something
		{
			this.DraggingFeatureIndex=Result.FeatureIndex;
			this.DraggingChildIndex=Result.MeshIndex;
			
			Used=true;
			TheEvent.stopPropagation();
		}
	}
	
	return(Used); 
}
/*
*/
CM3LayerVector.prototype.MouseMove=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
	
	if (this.DraggingFeatureIndex!=-1) // dragging a feature
	{
		this.GetFeatureMeshes();
		
//		for (var FeatureIndex=0;(this.TheFeatureMeshes!=null)&&(FeatureIndex<this.TheFeatureMeshes.length);FeatureIndex++)
		{
			var TheMesh=this.TheFeatureMeshes[this.DraggingFeatureIndex];
			
//			for (var j=0;j<TheMesh.children.length;j++)
			{
//				var TheChild=TheMesh.children[j];
				
//				var SelectedObjects=TheRayTracer.intersectObject(TheChild,false); // find the objects the ray intersects with
				
//				if (SelectedObjects.length>0)
				{
					if (TheRayTracer.ray.intersectPlane(this.DraggingPlane, this.IntersectionPoint))  // jjg - this appears to be undocumented!
					{
						var NewPosition=new THREE.Vector3();
						NewPosition.copy(this.IntersectionPoint);
//						NewPosition.sub(this.Anchor);
						
						var TheDataset=this.GetDataset();
						
//						TheDataset.SetPosition(this.DraggingFeatureIndex,0,0,0,NewPosition);
						TheDataset.SetSetting("Item","PositionOffset",[this.DraggingFeatureIndex,0,0,0,NewPosition]); 
					
						var TheChild=TheMesh.children[this.DraggingChildIndex];
				
						TheChild.position.copy(NewPosition);
		
						TheEvent.stopPropagation();
						Used=true;
					}
				}
			}
		}
	}
	else
	{
		if (this.TheFeatureMeshes!=null)
		{
			for (var FeatureIndex=0;FeatureIndex<this.TheFeatureMeshes.length;FeatureIndex++)
			{
				var TheMesh=this.TheFeatureMeshes[FeatureIndex];
				
				if (TheMesh!=null)
				{
					for (var j=0;j<TheMesh.children.length;j++)
					{
						var TheChild=TheMesh.children[j];
						
						if (TheChild!=null)
						{
							var SelectedObjects=TheRayTracer.intersectObject(TheChild,false); // find the objects the ray intersects with
							
							if (SelectedObjects.length>0)
							{
								if (TheRayTracer.ray.intersectPlane(this.DraggingPlane, this.IntersectionPoint))  // jjg - this appears to be undocumented!
								{
									var TheCamera=TheView.GetCamera();
									
									this.DraggingPlane.setFromNormalAndCoplanarPoint(
										TheCamera.getWorldDirection(this.DraggingPlane.normal), // normal to the this.DraggingPlane
										TheChild.position); // point on the this.DraggingPlane (coplanar point)
									
				//					TheEvent.stopPropagation();
				//					Used=true;
								}
							}
						}
					}
				}
			}
		}
	}
	
	return(Used); // used=true
}

/*
*/
CM3LayerVector.prototype.MouseUp=function(TheView,TheEvent,TheRayTracer)
{
	var Used=false;
	
	if (this.DraggingFeatureIndex!=-1)
	{
		this.DraggingFeatureIndex=-1;
		
		Used=true;
		TheEvent.stopPropagation();
	}
	
	return(Used); // used=true
}


//**************************************************************************************************
// CMLayer Functions
//**************************************************************************************************

CM3LayerVector.prototype.CMLayerDataset_OnLoad=CMLayerDataset.prototype.OnLoad;

CM3LayerVector.prototype.OnLoad=function()
{
	this.SettingsDirty=true;
	
	this.CMLayerDataset_OnLoad();
}
//**************************************************************************************************
// CM3LayerVector Functions
//**************************************************************************************************
/*
* Set a custom material to use to render the layer
* 
* @public
*/
CM3LayerVector.prototype.SetMaterial=function(TheMaterial)
{
	this.SettingsDirty=true;
	
	this.TheMaterial=TheMaterial;
}
/*
* Set an array of materials with one material for each feature
* @public
*/
CM3LayerVector.prototype.SetFeatureMaterials=function(TheFeatureMaterials)
{
	this.SettingsDirty=true;
	
	this.TheFeatureMaterials=TheFeatureMaterials;
}
	
/*
* Set the URL for a terrain (DEM) url for the vector data to follow
* @public
*/
CM3LayerVector.prototype.SetTerrainURL=function(TerrainURL,DataSetType) // jjg should be a setting
{
	//**********************************************************************
	// create the new data set or get an existing one
	if (this.TheTerrainDataset==null)
	{
		this.TheTerrainDataset=CMDataset.GetDataObject(TerrainURL,CMDataset.RASTER);
									 
		// Call the layer's OnLoad() function when the dataset loads the data
		this.TheTerrainDataset.AddListener(CMDataset.MESSAGE_DATASET_LOADED,this,function(TheDataset,ThisLayer,AdditionalInfo)
		{
			ThisLayer.UpdateMeshInGeo(false); // remove the meshes from the geo
			
			ThisLayer.TheFeatureMeshes=null;
			
			ThisLayer.GetFeatureMeshes(); // create the meshes
			
			ThisLayer.UpdateMeshInGeo(); // get the feature meshes and put them back into the OGL Scene if this layer is visible
		});
		this.TheTerrainDataset.SetURL(TerrainURL);
	}
}
/*
* Get the cross-sectional array to use to create extrusions for points and polylines
* @public
*/
CM3LayerVector.prototype.GetCrossSectionArray=function()
{
	if (this.CrossSectionArray==undefined)
	{
		var NumFaces=this.GetSetting("CrossSection","NumFaces",4);
		var Radius=this.GetSetting("CrossSection","Radius",300);
		
		this.CrossSectionArray=CM3Utilities.GetArcArrayInDegrees(0,360,NumFaces,Radius); // create the shape of the extrusion (start, end, segments, radius)
	}
	return(this.CrossSectionArray);
}
/*
* Set a cross-sectional array to use to create extrusions for points and polylines
* @public
*/
CM3LayerVector.prototype.SetCrossSectionArray=function(CrossSectionArray)
{
	this.CrossSectionArray=CrossSectionArray;
}
/*
* Set a custom CM3PathExtrusion object to use with polylines.
* @public
*/
CM3LayerVector.prototype.SetPathExtrusion=function(NewPathExtrusion)
{
	this.ThePathExtrusion=NewPathExtrusion;
}
/*
* Get the CM3PathExtrusion object to use with polylines.
* @public
*/
CM3LayerVector.prototype.GetPathExtrusion=function()
{
/*	if (this.ThePathExtrusion==undefined)
	{
		var CrossSectionArray=this.GetCrossSectionArray();
	
		var NumFaces=this.GetSetting("CrossSection","NumFaces",4);
		var Radius=this.GetSetting("CrossSection","Radius",300);
		
		var ExtrusionSettings=this.GetSettingGroup("PathExtrusion");
		
		// get the extruded polyline geometry
		this.ThePathExtrusion=new CM3PathExtrusion();
		this.ThePathExtrusion.SetCrossSectionArray(CrossSectionArray);
		this.ThePathExtrusion.SetSettingGroup("PathExtrusion",ExtrusionSettings);
	}
*/	return(this.ThePathExtrusion);
}
