/**************************************************************************************************
* CM3ProjectorSpherical Class
* Projector to convert geographic coordinates to and from spherial (x,y,z) coordinates)
*
* @module CM3ProjectorSpherical
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//***************************************************************************************
//	CMProjector class to convert between Geographic and UTM coordinates
//***************************************************************************************

//******************************************************************
// Constructor
//******************************************************************
/*
* Constructor
* @public, @constructs
*/
function CM3ProjectorSpherical() 
{
	CMProjector.call(this);

}
CM3ProjectorSpherical.prototype=Object.create(CMProjector.prototype); // inherit prototype functions from PanelBase()

CM3ProjectorSpherical.prototype.contructor=CM3ProjectorSpherical; // override the constructor to go to ours

CM3ProjectorSpherical.Radius=6371;

//***************************************************************************************
//	CMProjector functions
//***************************************************************************************
/**
*
*/
CM3ProjectorSpherical.prototype.GetBounds=function()
{
	var Bounds={
		XMin:-180, 
		XMax:180, 
		YMin:-90,
		YMax:90,
	};
	return(Bounds);
}
CM3ProjectorSpherical.prototype.GetClippingPolys=function()
{
	var Result=CMProjector.GetGeographicClippingPolys(-180,180,-90,90,0);
	
	return(Result);
}
//***************************************************************************************
//	CMProjector functions
//***************************************************************************************

//
// Projects a lat,long, elevation coordinate into a x,y,z coordinate in 3d space.
//	Result[0] - x
//	Result[1] - y
//	Result[2] - z
//
CM3ProjectorSpherical.prototype.ProjectFromGeographic=function(Long,Lat,Elevation)
{
	var LongInRadians=Long/180*Math.PI;
	var LatInRadians=Lat/180*Math.PI;
	
	var NewRadius=CM3ProjectorSpherical.Radius+Elevation;
	
	var x = NewRadius * Math.cos( LongInRadians ) * Math.cos( LatInRadians );
	var y = NewRadius * Math.sin( LongInRadians ) * Math.cos( LatInRadians );
	var z = NewRadius * Math.sin( LatInRadians );
	
	var Result=[x,y,z];
	
	return(Result);
}
/*
* Not finished
*/
CM3ProjectorSpherical.prototype.ProjectToGeographic=function(x,y,z)
{
	// jjg - not sure this is right 
	var lat = asin(z / CM3ProjectorSpherical.Radius)
	var lon = atan2(y, x)

	var Result=[Long,Lat,Elevation];
	
	return(Result);
}
//***************************************************************************************
//	CMProjector3D functions
//***************************************************************************************
/*
* Not finished
*/
CM3ProjectorSpherical.prototype.GetNormalFromGeographic=function(Long,Lat)
{
	var LongInRadians=Long/180*Math.PI;
	var LatInRadians=Lat/180*Math.PI;
	
	var x = Math.cos( LongInRadians ) * Math.cos( LatInRadians );
	var y = Math.sin( LongInRadians ) * Math.cos( LatInRadians );
	var z = Math.sin( LatInRadians );
	
	var TheVector=new THREE.Vector3(x,y,z);
	
	return(TheVector);
}
/**
* Returns the angles to rotate extruded content from the surface
*//*
CM3ProjectorSpherical.prototype.GetRotationsFromProjected=function(x,y,z)
{
		//var Vector1=TheGeo.GetAnglesFromProjected(x,y,z);
	
	//var radius=6371000; // r is know

	var AngleParaScreen=Math.asin(x/CM3ProjectorSpherical.Radius);// looks correct
	
	if (z<0)
	{
		AngleParaScreen=Math.PI-AngleParaScreen;
	}
	var AngleOrthagonalToScreen=-Math.asin(y/CM3ProjectorSpherical.Radius);
	if (z<0)
	{
		AngleOrthagonalToScreen=-AngleOrthagonalToScreen;
	}
	
	var TheVector=new THREE.Vector3(AngleOrthagonalToScreen,AngleParaScreen,0);

	return(TheVector);
}*/
CM3ProjectorSpherical.prototype.GetPositionedMesh=function(mesh1,Lon,Lat,ASL)
{
	// setup the two groups with the rotations required for lat/lon
	var LonInRadians=Lon/180*Math.PI;
	var LatInRadians=Lat/180*Math.PI;
	
	// rotate the mesh based on latitude (i.e. turn it vertical at the poles but horizontal at the equator)
	var LatRotationGroup = new THREE.Group();
	LatRotationGroup.rotation.set(0,LatInRadians-Math.PI/2,0);
	
	// position the mesh and add it to the rotated group
	mesh1.position.set( 0,0,CM3ProjectorSpherical.Radius+ASL); //
	LatRotationGroup.add( mesh1 );
	
	// create a group for the longitude rotation and add the latitude rotation group to it.
	group2=new THREE.Group();
	group2.add(LatRotationGroup);
	group2.rotation.set(0,0,LonInRadians-Math.PI);

	return(group2);
}