/**************************************************************************************************
* CM3SpriteText Class
*
* @module CM3SpriteText
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
CM3SpriteText=function(TheString)
{
	CM3Item.call(this);
	
	// settings
	this.TheString=TheString; // jjg - setting from CMItem?
	
	// other properties
	this.TheObject=null;
	
	this.SetSetting("Item","Scale",[50,50,1]); /// jjg ?
}
CM3SpriteText.prototype =Object.create(CM3Item.prototype);

CM3SpriteText.prototype.constructor = CM3SpriteText;

//******************************************************************
// Definitions
//******************************************************************

/**
* Below are the settings definitions.
* @public, @settings
*/
CM3SpriteText.SettingDefintions=
{
	Box:
	{
		VerticalPad: { Name:"Vertical Pad",Type:CMBase.DATA_TYPE_FLOAT, Default:3 },
		HorizontalPad: { Name:"Horizontal Pad",Type:CMBase.DATA_TYPE_FLOAT, Default:3 },
		Radius: { Name:"Corner Radius",Type:CMBase.DATA_TYPE_FLOAT, Default:3 },
	}
};

//******************************************************************
// Private functions
//******************************************************************
/** 
* Private function for drawing rounded rectangles
* @private
*/
CM3SpriteText.FollowRoundedRectPath=function(ctx, x, y, w, h, r) 
{
    ctx.beginPath();
    ctx.moveTo(x+r, y);
    ctx.lineTo(x+w-r, y);
    ctx.quadraticCurveTo(x+w, y, x+w, y+r);
    ctx.lineTo(x+w, y+h-r);
    ctx.quadraticCurveTo(x+w, y+h, x+w-r, y+h);
    ctx.lineTo(x+r, y+h);
    ctx.quadraticCurveTo(x, y+h, x, y+h-r);
    ctx.lineTo(x, y+r);
    ctx.quadraticCurveTo(x, y, x+r, y);
    ctx.closePath();
}

//**************************************************************************************************
// CM3Item Functions
//**************************************************************************************************
CM3SpriteText.prototype.GetGeometry=function()
{
}
CM3SpriteText.prototype.GetMaterial=function()
{
	var TheCanvasElement = document.createElement('canvas');
	var CanvasDimension=256;
	TheCanvasElement.width=CanvasDimension; // must be a factor of 2
	TheCanvasElement.height=CanvasDimension;

	var TheContext = TheCanvasElement.getContext('2d');
   
 	var TheStyle=this.GetStyle(null,0,"Text");
	CMView.SetStyleToContext(TheStyle,false,TheContext);

	// get the size of the characters
	var Font=this.GetSetting("Text","font","16px Arial");
	
	var FontSize=CMUtilities.GetFontSizeFromFont(Font);

	// get size data (height depends only on font size)
	var metrics = TheContext.measureText( this.TheString );
	var textWidth = metrics.width;
	
	// find the dimeions of the box
	// 1.4 is extra height factor for text below baseline: g,j,p,q.
	var VerticalPad=this.GetSetting("Box","VerticalPad",5);
	var HorizontalPad=this.GetSetting("Box","HorizontalPad",5);
	var Radius=this.GetSetting("Box","Radius",1);
	
	var BoxWidth=textWidth+HorizontalPad*2;
	var BoxHeight=FontSize * 1.4 + VerticalPad*2;
	
	var Left=(CanvasDimension-BoxWidth)/2;
	var Top=(CanvasDimension-BoxHeight)/2;
	
	if (false) // Debugging, outline the canvas sprite
	{
		TheContext.lineStyle = "blue";
		TheContext.strokeRect( 0, 0, TheCanvasElement.width,TheCanvasElement.height);
	}
	
	// darw the rounded rectangle around the text
 	var TheStyle=this.GetStyle(null,0,"Style");
	CMView.SetStyleToContext(TheStyle,false,TheContext);
	
	CM3SpriteText.FollowRoundedRectPath(TheContext,Left, Top, BoxWidth, BoxHeight, Radius);
	
	TheContext.fill();
	TheContext.stroke();   

	// paint the text
 	var TheStyle=this.GetStyle(null,0,"Text");
	CMView.SetStyleToContext(TheStyle,false,TheContext);

	TheContext.fillText( this.TheString, Left+HorizontalPad, Top+FontSize+VerticalPad);
	
	// Use the Canvas as a texture in the SpriteMaterial
	var texture = new THREE.Texture(TheCanvasElement) 
	texture.needsUpdate = true;

	var spriteMaterial = new THREE.SpriteMaterial( 
	{ 
		map: texture 
	} );
	return(spriteMaterial);	
}
/**
* Old code trying different approaches to creating text
*/
CM3SpriteText.prototype.Initialize=function()
{
	if ((this.TheOGLObject==null))
	{
		this.SetOGLObjectVisible(false); // make the object visible
		
		var LeftMaterial=this.GetMaterial();
		
		this.TheOGLObject = new THREE.Sprite( LeftMaterial );
	
		this.TheOGLObject.position.copy( this.ThePosition );	
		
		var Rotation=this.GetSetting("Item","Rotation");
		
		if (Rotation!=undefined)
		{
			this.TheOGLObject.rotateX(Rotation[0]);
			this.TheOGLObject.rotateY(Rotation[1]);
			this.TheOGLObject.rotateZ(Rotation[2]);
		}
		var Scale=this.GetSetting("Item","Scale");
		
		if (Scale!=undefined)
		{
			this.TheOGLObject.scale.set(Scale[0],Scale[1],Scale[2]);
		}
		
		this.TheOGLObject.TheCM3Item=this;
		
		this.SetOGLObjectVisible(true); // make the object visible
	}
};
