/**************************************************************************************************
* CM3Text Class
* Creates 3D objects of text including beveled text. See: https://threejs.org/docs/#api/en/geometries/TextGeometry
*
* @module CM3Text
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************
/*
* Constructor
* @public, @constructs
*/
function CM3Text(TheString)
{
	CM3Item.call(this);
	
	// settings
	this.text=TheString;
	
	this.SetName(TheString);
	
	this.TimeSlices[0].Settings.Font=	
	{
		size:12,
		GlobalFontIndex:0,
	};
	this.TimeSlices[0].Settings.Extrusion=	
	{
		height:4,
		curveSegments:12,
		bevelEnabled:false, 
		bevelSegments: 3, 
		bevelSize:.1, 
		bevelThickness:.1
	};
	// other properties
	this.TheObject=null;
	this.TheOGLObject=null;
}
CM3Text.prototype =Object.create(CM3Item.prototype);

CM3Text.prototype.constructor = CM3Text;
//******************************************************************
// Definitions
//******************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/

CM3Text.SettingDefintions=
{
	Font:
	{
		size: { Name:"Size",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // size of the text
		GlobalFontIndex: { Name:"Global Font Index",Type:CMBase.DATA_TYPE_INTEGER, Default:0 }, // global font to use
	},
	Extrusion:
	{
		height: { Name:"height",Type:CMBase.DATA_TYPE_FLOAT, Default:2 }, // amount to extrude the object
		curveSegments: { Name:"curveSegments",Type:CMBase.DATA_TYPE_FLOAT, Default:12 }, // number of points on the curve
		bevelEnabled: { Name:"bevelEnabled",Type:CMBase.DATA_TYPE_BOOLEAN, Default:true }, // beveled instead of 90 degree change, keep true for good looking text
		bevelSegments: { Name:"bevelSegments",Type:CMBase.DATA_TYPE_FLOAT, Default:5 }, // number of bevel layers (more makes a smoother bevel but draws slower
		bevelSize: { Name:"bevelSize",Type:CMBase.DATA_TYPE_FLOAT, Default:0.1 },// width of bevel (distance from shape outline), keep > 0
		bevelThickness: { Name:"bevelThickness",Type:CMBase.DATA_TYPE_FLOAT, Default:0.1 }, // how deep the bevel goes into the extrusion ,keep > 0
	},
}

//**************************************************************************************************
// Global font handling
//**************************************************************************************************
CM3Text.GlobalFonts=[];

CM3Text.Loader = new THREE.FontLoader();
CM3Text.Callback=null;

CM3Text.LoadFonts=function(Which,Callback)
{
	if (Which instanceof String)
	{
		CM3Text.Loader.load(Which, function ( font ) {
			CM3Text.GlobalFonts.push(font);
			Callback(CM3Text.GlobalFonts.length-1); // return an index to the font
		} );
	}
	else
	{
		var FontCount=0;
		for (var i=0;i<Which.length;i++)
		{
			var URL=Which[i].URL;
			
			CM3Text.Callback=function(Callback,font) // Callback is added by the "bind()" call below
			{
				CM3Text.GlobalFonts.push(font);
				FontCount++;
				if (FontCount==Which.length)
				{
					Callback();
				}
			}.bind(this,Callback); // this works and adds the parameters to the start of the parameter list
			
			CM3Text.Loader.load( URL,CM3Text.Callback); 
		}
	}
}

//**************************************************************************************************
// Constructor
//**************************************************************************************************
CM3Text.prototype.GetGeometry=function()
{
	var TheGeometry=null;
	
	if ((CM3Text.GlobalFonts.length>0))
	{
		var GlobalFontIndex=this.GetSetting("Font","GlobalFontIndex",0);
		var size=this.GetSetting("Font","size",12);
		
		var height=this.GetSetting("Extrusion","height",1);
		var curveSegments=this.GetSetting("Extrusion","curveSegments",10);
		//var bevelEnabled=this.GetSetting("Extrusion","bevelEnabled",true);
		var bevelSegments=this.GetSetting("Extrusion","bevelSegments",5);
		var bevelSize=this.GetSetting("Extrusion","bevelSize",0.1);
		var bevelThickness=this.GetSetting("Extrusion","bevelThickness",0.1);
	
		var Settings = { 
			font: CM3Text.GlobalFonts[GlobalFontIndex],
			size: size,
			height: height,
			curveSegments: curveSegments,
			bevelEnabled: true,  // beveled instead of 90 degree change
			bevelSegments: bevelSegments,  // number of bevel layers (more makes a smoother bevel but draws slower
			bevelSize: bevelSize,  // width of bevel (distance from shape outline
			bevelThickness: bevelThickness // how deep the bevel goes into the extrusion 
	//		material: 0, // not supported
	//		extrudeMaterial: 1// not supported
		};
		TheGeometry=new THREE.TextBufferGeometry( this.text, Settings);

		TheGeometry.computeBoundingBox();
		TheGeometry.computeVertexNormals();
	}
	return(TheGeometry);
}
/**
* Old code trying different approaches to creating text
*/
CM3Text.prototype.Initialize2=function()
{
	if ((this.TheOGLObject==null)&&(CM3Text.GlobalFonts.length>0))
	{
		this.SetOGLObjectVisible(false); // make the object visible
		
		var textGeo=this.GetGeometry();
		
		var LeftMaterial=this.GetMaterial();
		
		this.TheOGLObject = new THREE.Mesh( textGeo, LeftMaterial );
	
		this.TheOGLObject.position.copy( this.ThePosition );	
		
		this.SetOGLObjectVisible(true); // make the object visible
	}
};
